#ifndef MXODBC_H
#define MXODBC_H

/*=========================================================================

  mxODBC -- A generic ODBC 2.0 interface for Python

  (c) 1998, Copyright Marc-Andre Lemburg; All Rights Reserved;
  See the documentation for further copyright information or contact
  the author (mailto:mal@lemburg.com).

 -------------------------------------------------------------------------*/

/*
  -------------------------------------------------------------------------
  Customizing section -- adjust as needed !
  -------------------------------------------------------------------------


  NOTE: All these defines can also be passed to the module via
  -DDefine_this_Symbol flags in the Setup file. This is highly
  recommended since it allows for a more flexible installation. */


/* Define this if you have problems with LONGVARCHARS being truncated
   or received with inserted \0 characters. */

/*#define ADABAS_WORKAROUND*/

/* Uncomment the following lines according to your ODBC driver's
   capabilities. */

/*#define DONT_HAVE_SQLTables*/
/*#define DONT_HAVE_SQLTablePrivileges*/
/*#define DONT_HAVE_SQLColumns*/
/*#define DONT_HAVE_SQLColumnPrivileges*/
/*#define DONT_HAVE_SQLForeignKeys*/
/*#define DONT_HAVE_SQLPrimaryKeys*/
/*#define DONT_HAVE_SQLProcedures*/
/*#define DONT_HAVE_SQLProcedureColumns*/
/*#define DONT_HAVE_SQLSpecialColumns*/
/*#define DONT_HAVE_SQLStatistics*/
/*#define DONT_HAVE_SQLDescribeParam*/
/*#define DONT_HAVE_SQLGetTypeInfo*/

/* Connecting to a database via a ODBC manager is usually done through
   a special API called SQLDriverConnect(). If you are compiling the
   interface against a ODBC manager you should define
   HAVE_SQLDriverConnect to enable the DriverConnect() API provided by
   mxODBC. */

/*#define HAVE_SQLDriverConnect*/

/* To have the mxODBC interface automatically check the driver's
   capabilities (well, at least in some cases...), it can use the
   SQLGetFunctions() API. This API must be provided by ODBC Managers
   and is optional for ODBC drivers. This switch tells mxODBC whether
   to try to use it or not. */

/*#define HAVE_SQLGetFunctions*/

/* Some ODBC drivers or underlying database don't support
   transactions.  This causes clearing AUTOCOMMIT at connection
   creation time to fail. You can change the default value for the
   constructor flag clear_auto_commit to 0 by defining the switch
   below: */

/*#define DONT_CLEAR_AUTOCOMMIT*/

/* To disable the use of transactions altogether, use the following
   switch (implies DONT_CLEAR_AUTOCOMMIT): */

/*#define DONT_HAVE_TRANSACTIONS*/

#if defined(DONT_HAVE_TRANSACTIONS) && !defined(DONT_CLEAR_AUTOCOMMIT)
# define DONT_CLEAR_AUTOCOMMIT
#endif

/* Some databases have support for multiple result sets. mxODBC
   currently doesn't, so we always define this switch. The
   cursor.nextset() method is disabled by this switch. */

#define DONT_HAVE_MULTIPLE_RESULT_SETS

/* Another quirk that might be of importance to you: ODBC defines
   timestamps to use nanoseconds for the fraction part while SQL seems
   to have evolved into using microseconds. This obviously leads to
   confusion and at least some ODBC drivers have bugs that cause the
   stored values to come back truncated or rescaled. For this reason
   mxODBC offers a possibility to change the scale to microseconds: */

/*#define USE_MICROSECOND_FRACTIONS*/

/* ANSI SQL defines that column and table names may have at most 18
   characters (a column may be described using <table>.<column> which
   makes 37 characters in total + 1 for the terminating 0-byte).  Some
   databases also allow longer names, so I have included this limit as
   an adjustable parameter. Set the value to 1 + max. length in
   characters. */

#ifndef MAX_COLUMN_NAME_LENGTH
# define MAX_COLUMN_NAME_LENGTH 38
#endif

/* Since some drivers have bugs that prevent cached SQL statements to work
   correctly, there is a compile time switch to turn off cursor.execute()
   caching: */

/*#define DISABLE_EXECUTE_CACHE */

/* mxODBC can bind input variables using the information from the
   driver/manager or by looking at the passed in Python types.
   
   Default is to use the driver/manager information
   (USE_SQLTYPE_BINDING).  This only works if the SQLDescribeParam()
   API is available (this is corrected in mxODBC.c in case the
   settings are wrong), which it is for most drivers and managers.

   Uncomment on eof the following lines to choose the binding
   method. */

/*#define USE_SQLTYPE_BINDING*/
/*#define USE_PYTHONTYPE_BINDING*/

/* ODBC states that SQL API calls can generate more than one error
   message which are put on a stack for the application to read.  The
   SQLError() API pops the top-most entry off of this stack. 

   mxODBC normally only reads the first entry in this stack. All other
   messages are ignored and should be deleted by the ODBC prior to the
   next SQL API call...  yet some drivers have problems with this.

   Define this switch to have mxODBC explicitly pop all error messages
   off the stack. Only the first will be used to format the exception
   code and text, the others will be logged in the debug log if
   enabled. */

/*#define FLUSH_SQL_ERRORS*/

/*-------------------------------------------------------------------------
  Include the SQL header files. You may have to edit these to point to
  the right files for your DB.
  -------------------------------------------------------------------------*/

#if defined(iODBC) || defined(MySQL)

/* iODBC include files needed by the MyODBC driver as well as iODBC: */
# include "isql.h"
# include "isqlext.h"

#ifdef _INTRINSIC_SQL_H
/* Versions of iODBC prior to the take over by OpenLink need these
   additional include files: */
# include "odbc_types.h"
# include "odbc_funcs.h"
#else
/* Version 2.50.2 of the OpenLink iODBC version messes up the
   definition of HENV, HDBC and HSTMT causing lots of warnings. This
   should correct it... */
# define HENV SQLHENV
# define HDBC SQLHDBC
# define HSTMT SQLHSTMT
#endif

# ifdef MySQL
/* MySQL specific defines: */
#  define MXODBC_INTERFACENAME "MySQL ODBC"
#  ifndef HAVE_SQLDriverConnect
#   define HAVE_SQLDriverConnect
#  endif
#  define USE_PYTHONTYPE_BINDING
# else
#  define MXODBC_INTERFACENAME "iODBC"
/* iODBC specific defines: */
#  ifndef HAVE_SQLDriverConnect
#   define HAVE_SQLDriverConnect
#  endif
#  ifndef HAVE_SQLGetFunctions
#   define HAVE_SQLGetFunctions
#  endif
# endif
#else

#ifdef SolidServer
/* Solid Server include files: */
# include "cli0core.h"
# include "cli0ext1.h"
# define MXODBC_INTERFACENAME "Solid ODBC"
#else

#ifdef Intersolv
/* Intersolv driver include files; probably not all needed... */
# include "sqlunx.h"
# include "sqltypes.h"
# include "sqlext.h"
# include "sql.h"
# include "odbcinst.h"
# define MXODBC_INTERFACENAME "Intersolv ODBC"
#else

#ifdef Informix
/* Informix driver included in the Informix SDK */
# include "infxcli.h"
# define MXODBC_INTERFACENAME "Informix ODBC"
#else

#ifdef OpenLink
/* OpenLink Unix ODBC driver; these user the iODBC manager on Unix */
# include "iodbc.h"
# include "isql.h"
# include "isqlext.h"
# include "udbcext.h"
# define MXODBC_INTERFACENAME "OpenLink ODBC"
# ifndef HAVE_SQLDriverConnect
#  define HAVE_SQLDriverConnect
# endif
#else

#ifdef Adabas
/* Adabas ODBC driver included in the SuSE Linux edition: */
# undef min 
# undef max
# include "WINDOWS.H"
# include "sql.h"
# include "sqlext.h"
# define MXODBC_INTERFACENAME "Adabas ODBC"
# ifndef ADABAS_WORKAROUND
#  define ADABAS_WORKAROUND
# endif
# ifndef USE_MICROSECOND_FRACTIONS
#  define USE_MICROSECOND_FRACTIONS
# endif
# ifndef HAVE_SQLDriverConnect
#  define HAVE_SQLDriverConnect
# endif
#else

#ifdef EasySoft
/* EasySoft ODBC-ODBC Bridge: */
# include "sql.h"
# include "sqlext.h"
# define MXODBC_INTERFACENAME "EasySoft ODBC"
# ifndef HAVE_SQLDriverConnect
#  define HAVE_SQLDriverConnect
# endif
# ifndef HAVE_SQLGetFunctions
#  define HAVE_SQLGetFunctions
# endif
/* This seems to be missing in their header files... */
typedef unsigned int HWND;
#else

/* Sanity check for proper Setup. */
#ifdef MISSING_SETUP
# error "*** Please edit the Setup file before running 'make'. ***"
#else

/* Default:
   Include files provided by the freely available Microsoft ODBC SDK */

/* WINDOWS.H redefines these without checking first... */
# undef min 
# undef max
# include "WINDOWS.H"
# include "sql.h"
# include "sqlext.h"

/* On Windows we always interface to the MS ODBC Manager... */
#ifdef MS_WINDOWS
# ifndef MS_ODBC_MANAGER
#  define MS_ODBC_MANAGER
# endif
#endif

/* Correct the interface name for the MS ODBC Manager */
#ifdef MS_ODBC_MANAGER
# define MXODBC_INTERFACENAME "MS ODBC Manager"
# ifndef HAVE_SQLDriverConnect
#  define HAVE_SQLDriverConnect
# endif
# ifndef HAVE_SQLGetFunctions
#  define HAVE_SQLGetFunctions
# endif
#else
# define MXODBC_INTERFACENAME "ODBC"
#endif

#endif /* MISSING_SETUP */
#endif /* EasySoft */
#endif /* Adabas */
#endif /* OpenLink */
#endif /* Informix */
#endif /* Intersolv */
#endif /* SolidServer */
#endif /* iODBC */

/*-------------------------------------------------------------------------
  No servicable parts below this line.
 -------------------------------------------------------------------------*/

/* For backward compatibility: */
#ifdef HAVE_ODBC_DRIVER_CONNECT
# define HAVE_SQLDriverConnect
#endif
#ifdef HAVE_SQLDriverConnect
# define HAVE_ODBC_DRIVER_CONNECT
#endif

/* The extension's name */
#define MXODBC_MODULE "mxODBC"

/* Include generic mx extension header file */
#include "mxh.h"

#ifdef __cplusplus
extern "C" {
#endif

/* --- Constants -----------------------------------------------*/

/* Date/Time conversion format values */
#define MXODBC_DATETIME_DATETIME 0 /* date/time/timestamp must be DateTime 
				      instances. This is the default for
				      in- and output. */
#define MXODBC_TUPLE_DATETIME	 1 /* date/time/timestamp must be tuples */
#define MXODBC_FLOAT_DATETIME	 2 /* date/time/timestamp must be floats */
#define MXODBC_STRING_DATETIME	 3 /* date/time/timestamp must be strings */
#define MXODBC_MAX_DATETIME	 3 /* maximum value allowed for this field */

/* Binding options: mxODBC can bind using the SQL type information it
   gets from the driver/manager or the Python parameter type
   information. */
#define MXODBC_BIND_USING_SQLTYPE	1 /* use the database column type */
#define MXODBC_BIND_USING_PYTHONTYPE	2 /* use the parameter type */
#define MXODBC_MAX_BIND_METHOD		2 /* maximum value allowed for
					     this field */

/* --- TypeDict Initializer structure --------------------------*/

typedef struct {
    char *sqlstring;		/* SQL type as string */
    int sqlcode;		/* SQL type as integer code */
} mxODBC_TypeCodeTranslation;

/* --- SQLDict Initializer structure ---------------------------*/

typedef struct {
    char *sqlstring;		/* SQL define as string */
    int sqlcode;		/* SQL define as integer code */
} mxODBC_SQLDefine;

/* --- ErrorDict Initializer structure --------------------------*/

typedef struct {
    char *sqlstate;		/* SQLState as string */
    PyObject **errorobj;	/* Address of exception object to use */
} mxODBC_ErrorCodeTranslation;

/* --- Database Connection Object -------------------------------*/

typedef struct {

    PyObject_HEAD
    HDBC hdbc;			/* Connection handle */
    short datetimeformat;	/* Date/Time format to be used per default */
    short closed;		/* Is the connection already closed ? */
    short bindmethod;		/* Variable input binding option; see
				   MXODBC_BIND_* */
    short have_SQLDescribeParam;/* Does the driver support
				   SQLDescribeParam ?  */

} mxODBCObject;

#ifdef MX_BUILDING_MXODBC
    MX_EXPORT(PyTypeObject) mxODBC_Type;
#else
    MX_IMPORT(PyTypeObject) mxODBC_Type;
#endif

#define mxODBC_Check(v) \
        (((mxODBCObject *)(v))->ob_type == &mxODBC_Type)

/* --- Database Cursor Object -----------------------------------*/

typedef struct _mxODBCursor_Variable {

    UCHAR colname[MAX_COLUMN_NAME_LENGTH];	/* Column name */
    SWORD colname_len;		/* Length of column name in bytes */
    SWORD sqltype;		/* Type code SQL_* */
    UDWORD sqllen;		/* Max. length of the data entry; this is
				   the amount of space that is allocated for
				   the output data buffer */
    SWORD sqlscale;		/* Scale of decimals */
    SWORD nullable;		/* NULLable ? */
    SWORD ctype;		/* C type to be used SQL_C_* */
    PyObject *(*convert)(struct _mxODBCursor_Variable *);
				/* C to Python converter */
    PTR data;			/* Data pointer */
    SDWORD data_len;		/* Data length (input and output) */
    SDWORD max_len;		/* Size the data buffer (data) */
    short use_getdata;		/* Don't bind this column, but use SQLGetData
				   and dynamic resizing instead */
    short free_data;		/* Do we have to call free(data) for cleanup ? */

} mxODBCursor_Variable;

typedef struct {

    PyObject_HEAD
    mxODBCObject *dbc;		/* Connection object for this cursor */
    HSTMT hstmt;		/* Statement handle */
    int colcount;		/* Number of result columns */
    int rowcount;		/* Number of result rows */
    int arraysize;		/* Array size (not used internally) */
    short datetimeformat;	/* Date/Time format to be used for
				   this cursor */
    mxODBCursor_Variable *vars;	/* Bound output variables (one per column) 
				   The following condition always holds:
				   If vars != NULL, then the array's size
				   is colcount. Don't change colcount after
				   vars is allocated -- the information
				   is needed for freeing the array again.
				 */
    PyObject *sqlcmd;		/* Reference to the last query string
				   or NULL */
    short closed;		/* Is the cursor closed ? */
    
} mxODBCursorObject;

#ifdef MX_BUILDING_MXODBC
    MX_EXPORT(PyTypeObject) mxODBCursor_Type;
#else
    MX_IMPORT(PyTypeObject) mxODBCursor_Type;
#endif

#define mxODBCursorCheck(v) \
        (((mxODBCursorObject *)(v))->ob_type == &mxODBCursor_Type)



/* EOF */
#ifdef __cplusplus
}
#endif
#endif
