/* 
  mxSQLCodes -- Interface to SQL defines through a Python class.

  (c) Marc-Andre Lemburg; All Rights Reserved; mailto: mal@lemburg.com
  See the documentation for further copyright information or contact
  the author.
*/

/* Define this to aid in finding memory leaks */
/*#define MAL_MEM_DEBUG*/
/*#define MAL_DEBUG*/

#include "mx.h"
#include "mxODBC.h"

/* --- Globals ------------------------------------------------------------ */

/* Intializer for the SQL class 

   Note: 

   The following symbols are not usable as SQL codes from Python:
    SQL_API, SQL_NULL_HENV, SQL_NULL_HDBC, SQL_NULL_HSTMT

   They have been removed from the output of the h-file
   extractor.
   
 */

static mxODBC_SQLDefine mxODBC_SQLDefines[] = {
#ifdef SQL_SPEC_MAJOR
    {"SPEC_MAJOR",    SQL_SPEC_MAJOR}, /* Major version of specification */
#endif
#ifdef SQL_SPEC_MINOR
    {"SPEC_MINOR",    SQL_SPEC_MINOR}, /* Minor version of specification */
#endif
#ifdef SQL_SQLSTATE_SIZE
    {"SQLSTATE_SIZE", SQL_SQLSTATE_SIZE}, /* size of SQLSTATE */
#endif
#ifdef SQL_MAX_MESSAGE_LENGTH
    {"MAX_MESSAGE_LENGTH", SQL_MAX_MESSAGE_LENGTH}, /* message buffer size */
#endif
#ifdef SQL_MAX_DSN_LENGTH
    {"MAX_DSN_LENGTH", SQL_MAX_DSN_LENGTH}, /* maximum data source name size */
#endif
#ifdef SQL_INVALID_HANDLE
    {"INVALID_HANDLE", SQL_INVALID_HANDLE},
#endif
#ifdef SQL_ERROR
    {"ERROR",         SQL_ERROR},
#endif
#ifdef SQL_SUCCESS
    {"SUCCESS",       SQL_SUCCESS},
#endif
#ifdef SQL_SUCCESS_WITH_INFO
    {"SUCCESS_WITH_INFO", SQL_SUCCESS_WITH_INFO},
#endif
#ifdef SQL_NO_DATA_FOUND
    {"NO_DATA_FOUND", SQL_NO_DATA_FOUND},
#endif
#ifdef SQL_CHAR
    {"CHAR",          SQL_CHAR},
#endif
#ifdef SQL_NUMERIC
    {"NUMERIC",       SQL_NUMERIC},
#endif
#ifdef SQL_DECIMAL
    {"DECIMAL",       SQL_DECIMAL},
#endif
#ifdef SQL_INTEGER
    {"INTEGER",       SQL_INTEGER},
#endif
#ifdef SQL_SMALLINT
    {"SMALLINT",      SQL_SMALLINT},
#endif
#ifdef SQL_FLOAT
    {"FLOAT",         SQL_FLOAT},
#endif
#ifdef SQL_REAL
    {"REAL",          SQL_REAL},
#endif
#ifdef SQL_DOUBLE
    {"DOUBLE",        SQL_DOUBLE},
#endif
#ifdef SQL_VARCHAR
    {"VARCHAR",       SQL_VARCHAR},
#endif
#ifdef SQL_TYPE_MIN
    {"TYPE_MIN",      SQL_TYPE_MIN},
#endif
#ifdef SQL_TYPE_NULL
    {"TYPE_NULL",     SQL_TYPE_NULL},
#endif
#ifdef SQL_TYPE_MAX
    {"TYPE_MAX",      SQL_TYPE_MAX},
#endif
#ifdef SQL_C_CHAR
    {"C_CHAR",        SQL_C_CHAR}, /* CHAR, VARCHAR, DECIMAL, NUMERIC */
#endif
#ifdef SQL_C_LONG
    {"C_LONG",        SQL_C_LONG}, /* INTEGER */
#endif
#ifdef SQL_C_SHORT
    {"C_SHORT",       SQL_C_SHORT}, /* SMALLINT */
#endif
#ifdef SQL_C_FLOAT
    {"C_FLOAT",       SQL_C_FLOAT}, /* REAL */
#endif
#ifdef SQL_C_DOUBLE
    {"C_DOUBLE",      SQL_C_DOUBLE}, /* FLOAT, DOUBLE */
#endif
#ifdef SQL_C_DEFAULT
    {"C_DEFAULT",     SQL_C_DEFAULT},
#endif
#ifdef SQL_NO_NULLS
    {"NO_NULLS",      SQL_NO_NULLS},
#endif
#ifdef SQL_NULLABLE
    {"NULLABLE",      SQL_NULLABLE},
#endif
#ifdef SQL_NULLABLE_UNKNOWN
    {"NULLABLE_UNKNOWN", SQL_NULLABLE_UNKNOWN},
#endif
#ifdef SQL_NULL_DATA
    {"NULL_DATA",     SQL_NULL_DATA},
#endif
#ifdef SQL_DATA_AT_EXEC
    {"DATA_AT_EXEC",  SQL_DATA_AT_EXEC},
#endif
#ifdef SQL_NTS
    {"NTS",           SQL_NTS},
#endif
#ifdef SQL_CLOSE
    {"CLOSE",         SQL_CLOSE},
#endif
#ifdef SQL_DROP
    {"DROP",          SQL_DROP},
#endif
#ifdef SQL_UNBIND
    {"UNBIND",        SQL_UNBIND},
#endif
#ifdef SQL_RESET_PARAMS
    {"RESET_PARAMS",  SQL_RESET_PARAMS},
#endif
#ifdef SQL_COMMIT
    {"COMMIT",        SQL_COMMIT},
#endif
#ifdef SQL_ROLLBACK
    {"ROLLBACK",      SQL_ROLLBACK},
#endif
#ifdef SQL_COLUMN_COUNT
    {"COLUMN_COUNT",  SQL_COLUMN_COUNT},
#endif
#ifdef SQL_COLUMN_NAME
    {"COLUMN_NAME",   SQL_COLUMN_NAME},
#endif
#ifdef SQL_COLUMN_TYPE
    {"COLUMN_TYPE",   SQL_COLUMN_TYPE},
#endif
#ifdef SQL_COLUMN_LENGTH
    {"COLUMN_LENGTH", SQL_COLUMN_LENGTH},
#endif
#ifdef SQL_COLUMN_PRECISION
    {"COLUMN_PRECISION", SQL_COLUMN_PRECISION},
#endif
#ifdef SQL_COLUMN_SCALE
    {"COLUMN_SCALE",  SQL_COLUMN_SCALE},
#endif
#ifdef SQL_COLUMN_DISPLAY_SIZE
    {"COLUMN_DISPLAY_SIZE", SQL_COLUMN_DISPLAY_SIZE},
#endif
#ifdef SQL_COLUMN_NULLABLE
    {"COLUMN_NULLABLE", SQL_COLUMN_NULLABLE},
#endif
#ifdef SQL_COLUMN_UNSIGNED
    {"COLUMN_UNSIGNED", SQL_COLUMN_UNSIGNED},
#endif
#ifdef SQL_COLUMN_MONEY
    {"COLUMN_MONEY",  SQL_COLUMN_MONEY},
#endif
#ifdef SQL_COLUMN_UPDATABLE
    {"COLUMN_UPDATABLE", SQL_COLUMN_UPDATABLE},
#endif
#ifdef SQL_COLUMN_AUTO_INCREMENT
    {"COLUMN_AUTO_INCREMENT", SQL_COLUMN_AUTO_INCREMENT},
#endif
#ifdef SQL_COLUMN_CASE_SENSITIVE
    {"COLUMN_CASE_SENSITIVE", SQL_COLUMN_CASE_SENSITIVE},
#endif
#ifdef SQL_COLUMN_SEARCHABLE
    {"COLUMN_SEARCHABLE", SQL_COLUMN_SEARCHABLE},
#endif
#ifdef SQL_COLUMN_TYPE_NAME
    {"COLUMN_TYPE_NAME", SQL_COLUMN_TYPE_NAME},
#endif
#ifdef SQL_COLUMN_TABLE_NAME
    {"COLUMN_TABLE_NAME", SQL_COLUMN_TABLE_NAME},
#endif
#ifdef SQL_COLUMN_OWNER_NAME
    {"COLUMN_OWNER_NAME", SQL_COLUMN_OWNER_NAME},
#endif
#ifdef SQL_COLUMN_QUALIFIER_NAME
    {"COLUMN_QUALIFIER_NAME", SQL_COLUMN_QUALIFIER_NAME},
#endif
#ifdef SQL_COLUMN_LABEL
    {"COLUMN_LABEL",  SQL_COLUMN_LABEL},
#endif
#ifdef SQL_COLATT_OPT_MAX
    {"COLATT_OPT_MAX", SQL_COLATT_OPT_MAX},
#endif
#ifdef SQL_COLUMN_DRIVER_START
    {"COLUMN_DRIVER_START", SQL_COLUMN_DRIVER_START},
#endif
#ifdef SQL_COLATT_OPT_MIN
    {"COLATT_OPT_MIN", SQL_COLATT_OPT_MIN},
#endif
#ifdef SQL_ATTR_READONLY
    {"ATTR_READONLY", SQL_ATTR_READONLY},
#endif
#ifdef SQL_ATTR_WRITE
    {"ATTR_WRITE",    SQL_ATTR_WRITE},
#endif
#ifdef SQL_ATTR_READWRITE_UNKNOWN
    {"ATTR_READWRITE_UNKNOWN", SQL_ATTR_READWRITE_UNKNOWN},
#endif
#ifdef SQL_UNSEARCHABLE
    {"UNSEARCHABLE",  SQL_UNSEARCHABLE},
#endif
#ifdef SQL_LIKE_ONLY
    {"LIKE_ONLY",     SQL_LIKE_ONLY},
#endif
#ifdef SQL_ALL_EXCEPT_LIKE
    {"ALL_EXCEPT_LIKE", SQL_ALL_EXCEPT_LIKE},
#endif
#ifdef SQL_SEARCHABLE
    {"SEARCHABLE",    SQL_SEARCHABLE},
#endif
#ifdef SQL_MAX_OPTION_STRING_LENGTH
    {"MAX_OPTION_STRING_LENGTH", SQL_MAX_OPTION_STRING_LENGTH},
#endif
#ifdef SQL_STILL_EXECUTING
    {"STILL_EXECUTING", SQL_STILL_EXECUTING},
#endif
#ifdef SQL_NEED_DATA
    {"NEED_DATA",     SQL_NEED_DATA},
#endif
#ifdef SQL_DATE
    {"DATE",          SQL_DATE},
#endif
#ifdef SQL_TIME
    {"TIME",          SQL_TIME},
#endif
#ifdef SQL_TIMESTAMP
    {"TIMESTAMP",     SQL_TIMESTAMP},
#endif
#ifdef SQL_LONGVARCHAR
    {"LONGVARCHAR",   SQL_LONGVARCHAR},
#endif
#ifdef SQL_BINARY
    {"BINARY",        SQL_BINARY},
#endif
#ifdef SQL_VARBINARY
    {"VARBINARY",     SQL_VARBINARY},
#endif
#ifdef SQL_LONGVARBINARY
    {"LONGVARBINARY", SQL_LONGVARBINARY},
#endif
#ifdef SQL_BIGINT
    {"BIGINT",        SQL_BIGINT},
#endif
#ifdef SQL_TINYINT
    {"TINYINT",       SQL_TINYINT},
#endif
#ifdef SQL_BIT
    {"BIT",           SQL_BIT},
#endif
#ifdef SQL_INTERVAL_YEAR
    {"INTERVAL_YEAR", SQL_INTERVAL_YEAR},
#endif
#ifdef SQL_INTERVAL_MONTH
    {"INTERVAL_MONTH", SQL_INTERVAL_MONTH},
#endif
#ifdef SQL_INTERVAL_YEAR_TO_MONTH
    {"INTERVAL_YEAR_TO_MONTH", SQL_INTERVAL_YEAR_TO_MONTH},
#endif
#ifdef SQL_INTERVAL_DAY
    {"INTERVAL_DAY",  SQL_INTERVAL_DAY},
#endif
#ifdef SQL_INTERVAL_HOUR
    {"INTERVAL_HOUR", SQL_INTERVAL_HOUR},
#endif
#ifdef SQL_INTERVAL_MINUTE
    {"INTERVAL_MINUTE", SQL_INTERVAL_MINUTE},
#endif
#ifdef SQL_INTERVAL_SECOND
    {"INTERVAL_SECOND", SQL_INTERVAL_SECOND},
#endif
#ifdef SQL_INTERVAL_DAY_TO_HOUR
    {"INTERVAL_DAY_TO_HOUR", SQL_INTERVAL_DAY_TO_HOUR},
#endif
#ifdef SQL_INTERVAL_DAY_TO_MINUTE
    {"INTERVAL_DAY_TO_MINUTE", SQL_INTERVAL_DAY_TO_MINUTE},
#endif
#ifdef SQL_INTERVAL_DAY_TO_SECOND
    {"INTERVAL_DAY_TO_SECOND", SQL_INTERVAL_DAY_TO_SECOND},
#endif
#ifdef SQL_INTERVAL_HOUR_TO_MINUTE
    {"INTERVAL_HOUR_TO_MINUTE", SQL_INTERVAL_HOUR_TO_MINUTE},
#endif
#ifdef SQL_INTERVAL_HOUR_TO_SECOND
    {"INTERVAL_HOUR_TO_SECOND", SQL_INTERVAL_HOUR_TO_SECOND},
#endif
#ifdef SQL_INTERVAL_MINUTE_TO_SECOND
    {"INTERVAL_MINUTE_TO_SECOND", SQL_INTERVAL_MINUTE_TO_SECOND},
#endif
#ifdef SQL_UNICODE
    {"UNICODE",       SQL_UNICODE},
#endif
#ifdef SQL_TYPE_DRIVER_START
    {"TYPE_DRIVER_START", SQL_TYPE_DRIVER_START},
#endif
#ifdef SQL_TYPE_DRIVER_END
    {"TYPE_DRIVER_END", SQL_TYPE_DRIVER_END},
#endif
#ifdef SQL_SIGNED_OFFSET
    {"SIGNED_OFFSET", SQL_SIGNED_OFFSET},
#endif
#ifdef SQL_UNSIGNED_OFFSET
    {"UNSIGNED_OFFSET", SQL_UNSIGNED_OFFSET},
#endif
#ifdef SQL_C_DATE
    {"C_DATE",        SQL_C_DATE},
#endif
#ifdef SQL_C_TIME
    {"C_TIME",        SQL_C_TIME},
#endif
#ifdef SQL_C_TIMESTAMP
    {"C_TIMESTAMP",   SQL_C_TIMESTAMP},
#endif
#ifdef SQL_C_BINARY
    {"C_BINARY",      SQL_C_BINARY},
#endif
#ifdef SQL_C_BIT
    {"C_BIT",         SQL_C_BIT},
#endif
#ifdef SQL_C_TINYINT
    {"C_TINYINT",     SQL_C_TINYINT},
#endif
#ifdef SQL_C_SLONG
    {"C_SLONG",       SQL_C_SLONG},
#endif
#ifdef SQL_C_SSHORT
    {"C_SSHORT",      SQL_C_SSHORT},
#endif
#ifdef SQL_C_STINYINT
    {"C_STINYINT",    SQL_C_STINYINT},
#endif
#ifdef SQL_C_ULONG
    {"C_ULONG",       SQL_C_ULONG},
#endif
#ifdef SQL_C_USHORT
    {"C_USHORT",      SQL_C_USHORT},
#endif
#ifdef SQL_C_UTINYINT
    {"C_UTINYINT",    SQL_C_UTINYINT},
#endif
#ifdef SQL_C_BOOKMARK
    {"C_BOOKMARK",    SQL_C_BOOKMARK}, /* BOOKMARK */
#endif
#ifdef SQL_TYPE_MIN
    {"TYPE_MIN",      SQL_TYPE_MIN},
#endif
#ifdef SQL_ALL_TYPES
    {"ALL_TYPES",     SQL_ALL_TYPES},
#endif
#ifdef SQL_DRIVER_NOPROMPT
    {"DRIVER_NOPROMPT", SQL_DRIVER_NOPROMPT},
#endif
#ifdef SQL_DRIVER_COMPLETE
    {"DRIVER_COMPLETE", SQL_DRIVER_COMPLETE},
#endif
#ifdef SQL_DRIVER_PROMPT
    {"DRIVER_PROMPT", SQL_DRIVER_PROMPT},
#endif
#ifdef SQL_DRIVER_COMPLETE_REQUIRED
    {"DRIVER_COMPLETE_REQUIRED", SQL_DRIVER_COMPLETE_REQUIRED},
#endif
#ifdef SQL_NO_TOTAL
    {"NO_TOTAL",      SQL_NO_TOTAL},
#endif
#ifdef SQL_DEFAULT_PARAM
    {"DEFAULT_PARAM", SQL_DEFAULT_PARAM},
#endif
#ifdef SQL_IGNORE
    {"IGNORE",        SQL_IGNORE},
#endif
#ifdef SQL_LEN_DATA_AT_EXEC_OFFSET
    {"LEN_DATA_AT_EXEC_OFFSET", SQL_LEN_DATA_AT_EXEC_OFFSET},
#endif
#ifdef SQL_LEN_DATA_AT_EXE
    {"LEN_DATA_AT_EXE", SQL_LEN_DATA_AT_EXE},
#endif
#ifdef SQL_API_SQLALLOCCONNECT
    {"API_SQLALLOCCONNECT", SQL_API_SQLALLOCCONNECT}, /* Core Functions */
#endif
#ifdef SQL_API_SQLALLOCENV
    {"API_SQLALLOCENV", SQL_API_SQLALLOCENV},
#endif
#ifdef SQL_API_SQLALLOCSTMT
    {"API_SQLALLOCSTMT", SQL_API_SQLALLOCSTMT},
#endif
#ifdef SQL_API_SQLBINDCOL
    {"API_SQLBINDCOL", SQL_API_SQLBINDCOL},
#endif
#ifdef SQL_API_SQLCANCEL
    {"API_SQLCANCEL", SQL_API_SQLCANCEL},
#endif
#ifdef SQL_API_SQLCOLATTRIBUTES
    {"API_SQLCOLATTRIBUTES", SQL_API_SQLCOLATTRIBUTES},
#endif
#ifdef SQL_API_SQLCONNECT
    {"API_SQLCONNECT", SQL_API_SQLCONNECT},
#endif
#ifdef SQL_API_SQLDESCRIBECOL
    {"API_SQLDESCRIBECOL", SQL_API_SQLDESCRIBECOL},
#endif
#ifdef SQL_API_SQLDISCONNECT
    {"API_SQLDISCONNECT", SQL_API_SQLDISCONNECT},
#endif
#ifdef SQL_API_SQLERROR
    {"API_SQLERROR",  SQL_API_SQLERROR},
#endif
#ifdef SQL_API_SQLEXECDIRECT
    {"API_SQLEXECDIRECT", SQL_API_SQLEXECDIRECT},
#endif
#ifdef SQL_API_SQLEXECUTE
    {"API_SQLEXECUTE", SQL_API_SQLEXECUTE},
#endif
#ifdef SQL_API_SQLFETCH
    {"API_SQLFETCH",  SQL_API_SQLFETCH},
#endif
#ifdef SQL_API_SQLFREECONNECT
    {"API_SQLFREECONNECT", SQL_API_SQLFREECONNECT},
#endif
#ifdef SQL_API_SQLFREEENV
    {"API_SQLFREEENV", SQL_API_SQLFREEENV},
#endif
#ifdef SQL_API_SQLFREESTMT
    {"API_SQLFREESTMT", SQL_API_SQLFREESTMT},
#endif
#ifdef SQL_API_SQLGETCURSORNAME
    {"API_SQLGETCURSORNAME", SQL_API_SQLGETCURSORNAME},
#endif
#ifdef SQL_API_SQLNUMRESULTCOLS
    {"API_SQLNUMRESULTCOLS", SQL_API_SQLNUMRESULTCOLS},
#endif
#ifdef SQL_API_SQLPREPARE
    {"API_SQLPREPARE", SQL_API_SQLPREPARE},
#endif
#ifdef SQL_API_SQLROWCOUNT
    {"API_SQLROWCOUNT", SQL_API_SQLROWCOUNT},
#endif
#ifdef SQL_API_SQLSETCURSORNAME
    {"API_SQLSETCURSORNAME", SQL_API_SQLSETCURSORNAME},
#endif
#ifdef SQL_API_SQLSETPARAM
    {"API_SQLSETPARAM", SQL_API_SQLSETPARAM},
#endif
#ifdef SQL_API_SQLTRANSACT
    {"API_SQLTRANSACT", SQL_API_SQLTRANSACT},
#endif
#ifdef SQL_NUM_FUNCTIONS
    {"NUM_FUNCTIONS", SQL_NUM_FUNCTIONS},
#endif
#ifdef SQL_EXT_API_START
    {"EXT_API_START", SQL_EXT_API_START},
#endif
#ifdef SQL_API_SQLCOLUMNS
    {"API_SQLCOLUMNS", SQL_API_SQLCOLUMNS}, /* Level 1 Functions */
#endif
#ifdef SQL_API_SQLDRIVERCONNECT
    {"API_SQLDRIVERCONNECT", SQL_API_SQLDRIVERCONNECT},
#endif
#ifdef SQL_API_SQLGETCONNECTOPTION
    {"API_SQLGETCONNECTOPTION", SQL_API_SQLGETCONNECTOPTION},
#endif
#ifdef SQL_API_SQLGETDATA
    {"API_SQLGETDATA", SQL_API_SQLGETDATA},
#endif
#ifdef SQL_API_SQLGETFUNCTIONS
    {"API_SQLGETFUNCTIONS", SQL_API_SQLGETFUNCTIONS},
#endif
#ifdef SQL_API_SQLGETINFO
    {"API_SQLGETINFO", SQL_API_SQLGETINFO},
#endif
#ifdef SQL_API_SQLGETSTMTOPTION
    {"API_SQLGETSTMTOPTION", SQL_API_SQLGETSTMTOPTION},
#endif
#ifdef SQL_API_SQLGETTYPEINFO
    {"API_SQLGETTYPEINFO", SQL_API_SQLGETTYPEINFO},
#endif
#ifdef SQL_API_SQLPARAMDATA
    {"API_SQLPARAMDATA", SQL_API_SQLPARAMDATA},
#endif
#ifdef SQL_API_SQLPUTDATA
    {"API_SQLPUTDATA", SQL_API_SQLPUTDATA},
#endif
#ifdef SQL_API_SQLSETCONNECTOPTION
    {"API_SQLSETCONNECTOPTION", SQL_API_SQLSETCONNECTOPTION},
#endif
#ifdef SQL_API_SQLSETSTMTOPTION
    {"API_SQLSETSTMTOPTION", SQL_API_SQLSETSTMTOPTION},
#endif
#ifdef SQL_API_SQLSPECIALCOLUMNS
    {"API_SQLSPECIALCOLUMNS", SQL_API_SQLSPECIALCOLUMNS},
#endif
#ifdef SQL_API_SQLSTATISTICS
    {"API_SQLSTATISTICS", SQL_API_SQLSTATISTICS},
#endif
#ifdef SQL_API_SQLTABLES
    {"API_SQLTABLES", SQL_API_SQLTABLES},
#endif
#ifdef SQL_API_SQLBROWSECONNECT
    {"API_SQLBROWSECONNECT", SQL_API_SQLBROWSECONNECT}, /* Level 2 Functions */
#endif
#ifdef SQL_API_SQLCOLUMNPRIVILEGES
    {"API_SQLCOLUMNPRIVILEGES", SQL_API_SQLCOLUMNPRIVILEGES},
#endif
#ifdef SQL_API_SQLDATASOURCES
    {"API_SQLDATASOURCES", SQL_API_SQLDATASOURCES},
#endif
#ifdef SQL_API_SQLDESCRIBEPARAM
    {"API_SQLDESCRIBEPARAM", SQL_API_SQLDESCRIBEPARAM},
#endif
#ifdef SQL_API_SQLEXTENDEDFETCH
    {"API_SQLEXTENDEDFETCH", SQL_API_SQLEXTENDEDFETCH},
#endif
#ifdef SQL_API_SQLFOREIGNKEYS
    {"API_SQLFOREIGNKEYS", SQL_API_SQLFOREIGNKEYS},
#endif
#ifdef SQL_API_SQLMORERESULTS
    {"API_SQLMORERESULTS", SQL_API_SQLMORERESULTS},
#endif
#ifdef SQL_API_SQLNATIVESQL
    {"API_SQLNATIVESQL", SQL_API_SQLNATIVESQL},
#endif
#ifdef SQL_API_SQLNUMPARAMS
    {"API_SQLNUMPARAMS", SQL_API_SQLNUMPARAMS},
#endif
#ifdef SQL_API_SQLPARAMOPTIONS
    {"API_SQLPARAMOPTIONS", SQL_API_SQLPARAMOPTIONS},
#endif
#ifdef SQL_API_SQLPRIMARYKEYS
    {"API_SQLPRIMARYKEYS", SQL_API_SQLPRIMARYKEYS},
#endif
#ifdef SQL_API_SQLPROCEDURECOLUMNS
    {"API_SQLPROCEDURECOLUMNS", SQL_API_SQLPROCEDURECOLUMNS},
#endif
#ifdef SQL_API_SQLPROCEDURES
    {"API_SQLPROCEDURES", SQL_API_SQLPROCEDURES},
#endif
#ifdef SQL_API_SQLSETPOS
    {"API_SQLSETPOS", SQL_API_SQLSETPOS},
#endif
#ifdef SQL_API_SQLSETSCROLLOPTIONS
    {"API_SQLSETSCROLLOPTIONS", SQL_API_SQLSETSCROLLOPTIONS},
#endif
#ifdef SQL_API_SQLTABLEPRIVILEGES
    {"API_SQLTABLEPRIVILEGES", SQL_API_SQLTABLEPRIVILEGES},
#endif
#ifdef SQL_API_SQLDRIVERS
    {"API_SQLDRIVERS", SQL_API_SQLDRIVERS},
#endif
#ifdef SQL_API_SQLBINDPARAMETER
    {"API_SQLBINDPARAMETER", SQL_API_SQLBINDPARAMETER},
#endif
#ifdef SQL_EXT_API_LAST
    {"EXT_API_LAST",  SQL_EXT_API_LAST},
#endif
#ifdef SQL_API_ALL_FUNCTIONS
    {"API_ALL_FUNCTIONS", SQL_API_ALL_FUNCTIONS},
#endif
#ifdef SQL_NUM_EXTENSIONS
    {"NUM_EXTENSIONS", SQL_NUM_EXTENSIONS},
#endif
#ifdef SQL_API_LOADBYORDINAL
    {"API_LOADBYORDINAL", SQL_API_LOADBYORDINAL},
#endif
#ifdef SQL_INFO_FIRST
    {"INFO_FIRST",    SQL_INFO_FIRST},
#endif
#ifdef SQL_ACTIVE_CONNECTIONS
    {"ACTIVE_CONNECTIONS", SQL_ACTIVE_CONNECTIONS},
#endif
#ifdef SQL_ACTIVE_STATEMENTS
    {"ACTIVE_STATEMENTS", SQL_ACTIVE_STATEMENTS},
#endif
#ifdef SQL_DATA_SOURCE_NAME
    {"DATA_SOURCE_NAME", SQL_DATA_SOURCE_NAME},
#endif
#ifdef SQL_DRIVER_HDBC
    {"DRIVER_HDBC",   SQL_DRIVER_HDBC},
#endif
#ifdef SQL_DRIVER_HENV
    {"DRIVER_HENV",   SQL_DRIVER_HENV},
#endif
#ifdef SQL_DRIVER_HSTMT
    {"DRIVER_HSTMT",  SQL_DRIVER_HSTMT},
#endif
#ifdef SQL_DRIVER_NAME
    {"DRIVER_NAME",   SQL_DRIVER_NAME},
#endif
#ifdef SQL_DRIVER_VER
    {"DRIVER_VER",    SQL_DRIVER_VER},
#endif
#ifdef SQL_FETCH_DIRECTION
    {"FETCH_DIRECTION", SQL_FETCH_DIRECTION},
#endif
#ifdef SQL_ODBC_API_CONFORMANCE
    {"ODBC_API_CONFORMANCE", SQL_ODBC_API_CONFORMANCE},
#endif
#ifdef SQL_ODBC_VER
    {"ODBC_VER",      SQL_ODBC_VER},
#endif
#ifdef SQL_ROW_UPDATES
    {"ROW_UPDATES",   SQL_ROW_UPDATES},
#endif
#ifdef SQL_ODBC_SAG_CLI_CONFORMANCE
    {"ODBC_SAG_CLI_CONFORMANCE", SQL_ODBC_SAG_CLI_CONFORMANCE},
#endif
#ifdef SQL_SERVER_NAME
    {"SERVER_NAME",   SQL_SERVER_NAME},
#endif
#ifdef SQL_SEARCH_PATTERN_ESCAPE
    {"SEARCH_PATTERN_ESCAPE", SQL_SEARCH_PATTERN_ESCAPE},
#endif
#ifdef SQL_ODBC_SQL_CONFORMANCE
    {"ODBC_SQL_CONFORMANCE", SQL_ODBC_SQL_CONFORMANCE},
#endif
#ifdef SQL_DBMS_NAME
    {"DBMS_NAME",     SQL_DBMS_NAME},
#endif
#ifdef SQL_DBMS_VER
    {"DBMS_VER",      SQL_DBMS_VER},
#endif
#ifdef SQL_ACCESSIBLE_TABLES
    {"ACCESSIBLE_TABLES", SQL_ACCESSIBLE_TABLES},
#endif
#ifdef SQL_ACCESSIBLE_PROCEDURES
    {"ACCESSIBLE_PROCEDURES", SQL_ACCESSIBLE_PROCEDURES},
#endif
#ifdef SQL_PROCEDURES
    {"PROCEDURES",    SQL_PROCEDURES},
#endif
#ifdef SQL_CONCAT_NULL_BEHAVIOR
    {"CONCAT_NULL_BEHAVIOR", SQL_CONCAT_NULL_BEHAVIOR},
#endif
#ifdef SQL_CURSOR_COMMIT_BEHAVIOR
    {"CURSOR_COMMIT_BEHAVIOR", SQL_CURSOR_COMMIT_BEHAVIOR},
#endif
#ifdef SQL_CURSOR_ROLLBACK_BEHAVIOR
    {"CURSOR_ROLLBACK_BEHAVIOR", SQL_CURSOR_ROLLBACK_BEHAVIOR},
#endif
#ifdef SQL_DATA_SOURCE_READ_ONLY
    {"DATA_SOURCE_READ_ONLY", SQL_DATA_SOURCE_READ_ONLY},
#endif
#ifdef SQL_DEFAULT_TXN_ISOLATION
    {"DEFAULT_TXN_ISOLATION", SQL_DEFAULT_TXN_ISOLATION},
#endif
#ifdef SQL_EXPRESSIONS_IN_ORDERBY
    {"EXPRESSIONS_IN_ORDERBY", SQL_EXPRESSIONS_IN_ORDERBY},
#endif
#ifdef SQL_IDENTIFIER_CASE
    {"IDENTIFIER_CASE", SQL_IDENTIFIER_CASE},
#endif
#ifdef SQL_IDENTIFIER_QUOTE_CHAR
    {"IDENTIFIER_QUOTE_CHAR", SQL_IDENTIFIER_QUOTE_CHAR},
#endif
#ifdef SQL_MAX_COLUMN_NAME_LEN
    {"MAX_COLUMN_NAME_LEN", SQL_MAX_COLUMN_NAME_LEN},
#endif
#ifdef SQL_MAX_CURSOR_NAME_LEN
    {"MAX_CURSOR_NAME_LEN", SQL_MAX_CURSOR_NAME_LEN},
#endif
#ifdef SQL_MAX_OWNER_NAME_LEN
    {"MAX_OWNER_NAME_LEN", SQL_MAX_OWNER_NAME_LEN},
#endif
#ifdef SQL_MAX_PROCEDURE_NAME_LEN
    {"MAX_PROCEDURE_NAME_LEN", SQL_MAX_PROCEDURE_NAME_LEN},
#endif
#ifdef SQL_MAX_QUALIFIER_NAME_LEN
    {"MAX_QUALIFIER_NAME_LEN", SQL_MAX_QUALIFIER_NAME_LEN},
#endif
#ifdef SQL_MAX_TABLE_NAME_LEN
    {"MAX_TABLE_NAME_LEN", SQL_MAX_TABLE_NAME_LEN},
#endif
#ifdef SQL_MULT_RESULT_SETS
    {"MULT_RESULT_SETS", SQL_MULT_RESULT_SETS},
#endif
#ifdef SQL_MULTIPLE_ACTIVE_TXN
    {"MULTIPLE_ACTIVE_TXN", SQL_MULTIPLE_ACTIVE_TXN},
#endif
#ifdef SQL_OUTER_JOINS
    {"OUTER_JOINS",   SQL_OUTER_JOINS},
#endif
#ifdef SQL_OWNER_TERM
    {"OWNER_TERM",    SQL_OWNER_TERM},
#endif
#ifdef SQL_PROCEDURE_TERM
    {"PROCEDURE_TERM", SQL_PROCEDURE_TERM},
#endif
#ifdef SQL_QUALIFIER_NAME_SEPARATOR
    {"QUALIFIER_NAME_SEPARATOR", SQL_QUALIFIER_NAME_SEPARATOR},
#endif
#ifdef SQL_QUALIFIER_TERM
    {"QUALIFIER_TERM", SQL_QUALIFIER_TERM},
#endif
#ifdef SQL_SCROLL_CONCURRENCY
    {"SCROLL_CONCURRENCY", SQL_SCROLL_CONCURRENCY},
#endif
#ifdef SQL_SCROLL_OPTIONS
    {"SCROLL_OPTIONS", SQL_SCROLL_OPTIONS},
#endif
#ifdef SQL_TABLE_TERM
    {"TABLE_TERM",    SQL_TABLE_TERM},
#endif
#ifdef SQL_TXN_CAPABLE
    {"TXN_CAPABLE",   SQL_TXN_CAPABLE},
#endif
#ifdef SQL_USER_NAME
    {"USER_NAME",     SQL_USER_NAME},
#endif
#ifdef SQL_CONVERT_FUNCTIONS
    {"CONVERT_FUNCTIONS", SQL_CONVERT_FUNCTIONS},
#endif
#ifdef SQL_NUMERIC_FUNCTIONS
    {"NUMERIC_FUNCTIONS", SQL_NUMERIC_FUNCTIONS},
#endif
#ifdef SQL_STRING_FUNCTIONS
    {"STRING_FUNCTIONS", SQL_STRING_FUNCTIONS},
#endif
#ifdef SQL_SYSTEM_FUNCTIONS
    {"SYSTEM_FUNCTIONS", SQL_SYSTEM_FUNCTIONS},
#endif
#ifdef SQL_TIMEDATE_FUNCTIONS
    {"TIMEDATE_FUNCTIONS", SQL_TIMEDATE_FUNCTIONS},
#endif
#ifdef SQL_CONVERT_BIGINT
    {"CONVERT_BIGINT", SQL_CONVERT_BIGINT},
#endif
#ifdef SQL_CONVERT_BINARY
    {"CONVERT_BINARY", SQL_CONVERT_BINARY},
#endif
#ifdef SQL_CONVERT_BIT
    {"CONVERT_BIT",   SQL_CONVERT_BIT},
#endif
#ifdef SQL_CONVERT_CHAR
    {"CONVERT_CHAR",  SQL_CONVERT_CHAR},
#endif
#ifdef SQL_CONVERT_DATE
    {"CONVERT_DATE",  SQL_CONVERT_DATE},
#endif
#ifdef SQL_CONVERT_DECIMAL
    {"CONVERT_DECIMAL", SQL_CONVERT_DECIMAL},
#endif
#ifdef SQL_CONVERT_DOUBLE
    {"CONVERT_DOUBLE", SQL_CONVERT_DOUBLE},
#endif
#ifdef SQL_CONVERT_FLOAT
    {"CONVERT_FLOAT", SQL_CONVERT_FLOAT},
#endif
#ifdef SQL_CONVERT_INTEGER
    {"CONVERT_INTEGER", SQL_CONVERT_INTEGER},
#endif
#ifdef SQL_CONVERT_LONGVARCHAR
    {"CONVERT_LONGVARCHAR", SQL_CONVERT_LONGVARCHAR},
#endif
#ifdef SQL_CONVERT_NUMERIC
    {"CONVERT_NUMERIC", SQL_CONVERT_NUMERIC},
#endif
#ifdef SQL_CONVERT_REAL
    {"CONVERT_REAL",  SQL_CONVERT_REAL},
#endif
#ifdef SQL_CONVERT_SMALLINT
    {"CONVERT_SMALLINT", SQL_CONVERT_SMALLINT},
#endif
#ifdef SQL_CONVERT_TIME
    {"CONVERT_TIME",  SQL_CONVERT_TIME},
#endif
#ifdef SQL_CONVERT_TIMESTAMP
    {"CONVERT_TIMESTAMP", SQL_CONVERT_TIMESTAMP},
#endif
#ifdef SQL_CONVERT_TINYINT
    {"CONVERT_TINYINT", SQL_CONVERT_TINYINT},
#endif
#ifdef SQL_CONVERT_VARBINARY
    {"CONVERT_VARBINARY", SQL_CONVERT_VARBINARY},
#endif
#ifdef SQL_CONVERT_VARCHAR
    {"CONVERT_VARCHAR", SQL_CONVERT_VARCHAR},
#endif
#ifdef SQL_CONVERT_LONGVARBINARY
    {"CONVERT_LONGVARBINARY", SQL_CONVERT_LONGVARBINARY},
#endif
#ifdef SQL_TXN_ISOLATION_OPTION
    {"TXN_ISOLATION_OPTION", SQL_TXN_ISOLATION_OPTION},
#endif
#ifdef SQL_ODBC_SQL_OPT_IEF
    {"ODBC_SQL_OPT_IEF", SQL_ODBC_SQL_OPT_IEF},
#endif
#ifdef SQL_CORRELATION_NAME
    {"CORRELATION_NAME", SQL_CORRELATION_NAME},
#endif
#ifdef SQL_NON_NULLABLE_COLUMNS
    {"NON_NULLABLE_COLUMNS", SQL_NON_NULLABLE_COLUMNS},
#endif
#ifdef SQL_DRIVER_HLIB
    {"DRIVER_HLIB",   SQL_DRIVER_HLIB},
#endif
#ifdef SQL_DRIVER_ODBC_VER
    {"DRIVER_ODBC_VER", SQL_DRIVER_ODBC_VER},
#endif
#ifdef SQL_LOCK_TYPES
    {"LOCK_TYPES",    SQL_LOCK_TYPES},
#endif
#ifdef SQL_POS_OPERATIONS
    {"POS_OPERATIONS", SQL_POS_OPERATIONS},
#endif
#ifdef SQL_POSITIONED_STATEMENTS
    {"POSITIONED_STATEMENTS", SQL_POSITIONED_STATEMENTS},
#endif
#ifdef SQL_GETDATA_EXTENSIONS
    {"GETDATA_EXTENSIONS", SQL_GETDATA_EXTENSIONS},
#endif
#ifdef SQL_BOOKMARK_PERSISTENCE
    {"BOOKMARK_PERSISTENCE", SQL_BOOKMARK_PERSISTENCE},
#endif
#ifdef SQL_STATIC_SENSITIVITY
    {"STATIC_SENSITIVITY", SQL_STATIC_SENSITIVITY},
#endif
#ifdef SQL_FILE_USAGE
    {"FILE_USAGE",    SQL_FILE_USAGE},
#endif
#ifdef SQL_NULL_COLLATION
    {"NULL_COLLATION", SQL_NULL_COLLATION},
#endif
#ifdef SQL_ALTER_TABLE
    {"ALTER_TABLE",   SQL_ALTER_TABLE},
#endif
#ifdef SQL_COLUMN_ALIAS
    {"COLUMN_ALIAS",  SQL_COLUMN_ALIAS},
#endif
#ifdef SQL_GROUP_BY
    {"GROUP_BY",      SQL_GROUP_BY},
#endif
#ifdef SQL_KEYWORDS
    {"KEYWORDS",      SQL_KEYWORDS},
#endif
#ifdef SQL_ORDER_BY_COLUMNS_IN_SELECT
    {"ORDER_BY_COLUMNS_IN_SELECT", SQL_ORDER_BY_COLUMNS_IN_SELECT},
#endif
#ifdef SQL_OWNER_USAGE
    {"OWNER_USAGE",   SQL_OWNER_USAGE},
#endif
#ifdef SQL_QUALIFIER_USAGE
    {"QUALIFIER_USAGE", SQL_QUALIFIER_USAGE},
#endif
#ifdef SQL_QUOTED_IDENTIFIER_CASE
    {"QUOTED_IDENTIFIER_CASE", SQL_QUOTED_IDENTIFIER_CASE},
#endif
#ifdef SQL_SPECIAL_CHARACTERS
    {"SPECIAL_CHARACTERS", SQL_SPECIAL_CHARACTERS},
#endif
#ifdef SQL_SUBQUERIES
    {"SUBQUERIES",    SQL_SUBQUERIES},
#endif
#ifdef SQL_UNION
    {"UNION",         SQL_UNION},
#endif
#ifdef SQL_MAX_COLUMNS_IN_GROUP_BY
    {"MAX_COLUMNS_IN_GROUP_BY", SQL_MAX_COLUMNS_IN_GROUP_BY},
#endif
#ifdef SQL_MAX_COLUMNS_IN_INDEX
    {"MAX_COLUMNS_IN_INDEX", SQL_MAX_COLUMNS_IN_INDEX},
#endif
#ifdef SQL_MAX_COLUMNS_IN_ORDER_BY
    {"MAX_COLUMNS_IN_ORDER_BY", SQL_MAX_COLUMNS_IN_ORDER_BY},
#endif
#ifdef SQL_MAX_COLUMNS_IN_SELECT
    {"MAX_COLUMNS_IN_SELECT", SQL_MAX_COLUMNS_IN_SELECT},
#endif
#ifdef SQL_MAX_COLUMNS_IN_TABLE
    {"MAX_COLUMNS_IN_TABLE", SQL_MAX_COLUMNS_IN_TABLE},
#endif
#ifdef SQL_MAX_INDEX_SIZE
    {"MAX_INDEX_SIZE", SQL_MAX_INDEX_SIZE},
#endif
#ifdef SQL_MAX_ROW_SIZE_INCLUDES_LONG
    {"MAX_ROW_SIZE_INCLUDES_LONG", SQL_MAX_ROW_SIZE_INCLUDES_LONG},
#endif
#ifdef SQL_MAX_ROW_SIZE
    {"MAX_ROW_SIZE",  SQL_MAX_ROW_SIZE},
#endif
#ifdef SQL_MAX_STATEMENT_LEN
    {"MAX_STATEMENT_LEN", SQL_MAX_STATEMENT_LEN},
#endif
#ifdef SQL_MAX_TABLES_IN_SELECT
    {"MAX_TABLES_IN_SELECT", SQL_MAX_TABLES_IN_SELECT},
#endif
#ifdef SQL_MAX_USER_NAME_LEN
    {"MAX_USER_NAME_LEN", SQL_MAX_USER_NAME_LEN},
#endif
#ifdef SQL_MAX_CHAR_LITERAL_LEN
    {"MAX_CHAR_LITERAL_LEN", SQL_MAX_CHAR_LITERAL_LEN},
#endif
#ifdef SQL_TIMEDATE_ADD_INTERVALS
    {"TIMEDATE_ADD_INTERVALS", SQL_TIMEDATE_ADD_INTERVALS},
#endif
#ifdef SQL_TIMEDATE_DIFF_INTERVALS
    {"TIMEDATE_DIFF_INTERVALS", SQL_TIMEDATE_DIFF_INTERVALS},
#endif
#ifdef SQL_NEED_LONG_DATA_LEN
    {"NEED_LONG_DATA_LEN", SQL_NEED_LONG_DATA_LEN},
#endif
#ifdef SQL_MAX_BINARY_LITERAL_LEN
    {"MAX_BINARY_LITERAL_LEN", SQL_MAX_BINARY_LITERAL_LEN},
#endif
#ifdef SQL_LIKE_ESCAPE_CLAUSE
    {"LIKE_ESCAPE_CLAUSE", SQL_LIKE_ESCAPE_CLAUSE},
#endif
#ifdef SQL_QUALIFIER_LOCATION
    {"QUALIFIER_LOCATION", SQL_QUALIFIER_LOCATION},
#endif
#ifdef SQL_OJ_CAPABILITIES
    {"OJ_CAPABILITIES", SQL_OJ_CAPABILITIES}, /* Temp value until ODBC 3.0 */
#endif
#ifdef SQL_INFO_LAST
    {"INFO_LAST",     SQL_INFO_LAST},
#endif
#ifdef SQL_INFO_DRIVER_START
    {"INFO_DRIVER_START", SQL_INFO_DRIVER_START},
#endif
#ifdef SQL_CVT_CHAR
    {"CVT_CHAR",      SQL_CVT_CHAR},
#endif
#ifdef SQL_CVT_NUMERIC
    {"CVT_NUMERIC",   SQL_CVT_NUMERIC},
#endif
#ifdef SQL_CVT_DECIMAL
    {"CVT_DECIMAL",   SQL_CVT_DECIMAL},
#endif
#ifdef SQL_CVT_INTEGER
    {"CVT_INTEGER",   SQL_CVT_INTEGER},
#endif
#ifdef SQL_CVT_SMALLINT
    {"CVT_SMALLINT",  SQL_CVT_SMALLINT},
#endif
#ifdef SQL_CVT_FLOAT
    {"CVT_FLOAT",     SQL_CVT_FLOAT},
#endif
#ifdef SQL_CVT_REAL
    {"CVT_REAL",      SQL_CVT_REAL},
#endif
#ifdef SQL_CVT_DOUBLE
    {"CVT_DOUBLE",    SQL_CVT_DOUBLE},
#endif
#ifdef SQL_CVT_VARCHAR
    {"CVT_VARCHAR",   SQL_CVT_VARCHAR},
#endif
#ifdef SQL_CVT_LONGVARCHAR
    {"CVT_LONGVARCHAR", SQL_CVT_LONGVARCHAR},
#endif
#ifdef SQL_CVT_BINARY
    {"CVT_BINARY",    SQL_CVT_BINARY},
#endif
#ifdef SQL_CVT_VARBINARY
    {"CVT_VARBINARY", SQL_CVT_VARBINARY},
#endif
#ifdef SQL_CVT_BIT
    {"CVT_BIT",       SQL_CVT_BIT},
#endif
#ifdef SQL_CVT_TINYINT
    {"CVT_TINYINT",   SQL_CVT_TINYINT},
#endif
#ifdef SQL_CVT_BIGINT
    {"CVT_BIGINT",    SQL_CVT_BIGINT},
#endif
#ifdef SQL_CVT_DATE
    {"CVT_DATE",      SQL_CVT_DATE},
#endif
#ifdef SQL_CVT_TIME
    {"CVT_TIME",      SQL_CVT_TIME},
#endif
#ifdef SQL_CVT_TIMESTAMP
    {"CVT_TIMESTAMP", SQL_CVT_TIMESTAMP},
#endif
#ifdef SQL_CVT_LONGVARBINARY
    {"CVT_LONGVARBINARY", SQL_CVT_LONGVARBINARY},
#endif
#ifdef SQL_FN_CVT_CONVERT
    {"FN_CVT_CONVERT", SQL_FN_CVT_CONVERT},
#endif
#ifdef SQL_FN_STR_CONCAT
    {"FN_STR_CONCAT", SQL_FN_STR_CONCAT},
#endif
#ifdef SQL_FN_STR_INSERT
    {"FN_STR_INSERT", SQL_FN_STR_INSERT},
#endif
#ifdef SQL_FN_STR_LEFT
    {"FN_STR_LEFT",   SQL_FN_STR_LEFT},
#endif
#ifdef SQL_FN_STR_LTRIM
    {"FN_STR_LTRIM",  SQL_FN_STR_LTRIM},
#endif
#ifdef SQL_FN_STR_LENGTH
    {"FN_STR_LENGTH", SQL_FN_STR_LENGTH},
#endif
#ifdef SQL_FN_STR_LOCATE
    {"FN_STR_LOCATE", SQL_FN_STR_LOCATE},
#endif
#ifdef SQL_FN_STR_LCASE
    {"FN_STR_LCASE",  SQL_FN_STR_LCASE},
#endif
#ifdef SQL_FN_STR_REPEAT
    {"FN_STR_REPEAT", SQL_FN_STR_REPEAT},
#endif
#ifdef SQL_FN_STR_REPLACE
    {"FN_STR_REPLACE", SQL_FN_STR_REPLACE},
#endif
#ifdef SQL_FN_STR_RIGHT
    {"FN_STR_RIGHT",  SQL_FN_STR_RIGHT},
#endif
#ifdef SQL_FN_STR_RTRIM
    {"FN_STR_RTRIM",  SQL_FN_STR_RTRIM},
#endif
#ifdef SQL_FN_STR_SUBSTRING
    {"FN_STR_SUBSTRING", SQL_FN_STR_SUBSTRING},
#endif
#ifdef SQL_FN_STR_UCASE
    {"FN_STR_UCASE",  SQL_FN_STR_UCASE},
#endif
#ifdef SQL_FN_STR_ASCII
    {"FN_STR_ASCII",  SQL_FN_STR_ASCII},
#endif
#ifdef SQL_FN_STR_CHAR
    {"FN_STR_CHAR",   SQL_FN_STR_CHAR},
#endif
#ifdef SQL_FN_STR_DIFFERENCE
    {"FN_STR_DIFFERENCE", SQL_FN_STR_DIFFERENCE},
#endif
#ifdef SQL_FN_STR_LOCATE_2
    {"FN_STR_LOCATE_2", SQL_FN_STR_LOCATE_2},
#endif
#ifdef SQL_FN_STR_SOUNDEX
    {"FN_STR_SOUNDEX", SQL_FN_STR_SOUNDEX},
#endif
#ifdef SQL_FN_STR_SPACE
    {"FN_STR_SPACE",  SQL_FN_STR_SPACE},
#endif
#ifdef SQL_FN_NUM_ABS
    {"FN_NUM_ABS",    SQL_FN_NUM_ABS},
#endif
#ifdef SQL_FN_NUM_ACOS
    {"FN_NUM_ACOS",   SQL_FN_NUM_ACOS},
#endif
#ifdef SQL_FN_NUM_ASIN
    {"FN_NUM_ASIN",   SQL_FN_NUM_ASIN},
#endif
#ifdef SQL_FN_NUM_ATAN
    {"FN_NUM_ATAN",   SQL_FN_NUM_ATAN},
#endif
#ifdef SQL_FN_NUM_ATAN2
    {"FN_NUM_ATAN2",  SQL_FN_NUM_ATAN2},
#endif
#ifdef SQL_FN_NUM_CEILING
    {"FN_NUM_CEILING", SQL_FN_NUM_CEILING},
#endif
#ifdef SQL_FN_NUM_COS
    {"FN_NUM_COS",    SQL_FN_NUM_COS},
#endif
#ifdef SQL_FN_NUM_COT
    {"FN_NUM_COT",    SQL_FN_NUM_COT},
#endif
#ifdef SQL_FN_NUM_EXP
    {"FN_NUM_EXP",    SQL_FN_NUM_EXP},
#endif
#ifdef SQL_FN_NUM_FLOOR
    {"FN_NUM_FLOOR",  SQL_FN_NUM_FLOOR},
#endif
#ifdef SQL_FN_NUM_LOG
    {"FN_NUM_LOG",    SQL_FN_NUM_LOG},
#endif
#ifdef SQL_FN_NUM_MOD
    {"FN_NUM_MOD",    SQL_FN_NUM_MOD},
#endif
#ifdef SQL_FN_NUM_SIGN
    {"FN_NUM_SIGN",   SQL_FN_NUM_SIGN},
#endif
#ifdef SQL_FN_NUM_SIN
    {"FN_NUM_SIN",    SQL_FN_NUM_SIN},
#endif
#ifdef SQL_FN_NUM_SQRT
    {"FN_NUM_SQRT",   SQL_FN_NUM_SQRT},
#endif
#ifdef SQL_FN_NUM_TAN
    {"FN_NUM_TAN",    SQL_FN_NUM_TAN},
#endif
#ifdef SQL_FN_NUM_PI
    {"FN_NUM_PI",     SQL_FN_NUM_PI},
#endif
#ifdef SQL_FN_NUM_RAND
    {"FN_NUM_RAND",   SQL_FN_NUM_RAND},
#endif
#ifdef SQL_FN_NUM_DEGREES
    {"FN_NUM_DEGREES", SQL_FN_NUM_DEGREES},
#endif
#ifdef SQL_FN_NUM_LOG10
    {"FN_NUM_LOG10",  SQL_FN_NUM_LOG10},
#endif
#ifdef SQL_FN_NUM_POWER
    {"FN_NUM_POWER",  SQL_FN_NUM_POWER},
#endif
#ifdef SQL_FN_NUM_RADIANS
    {"FN_NUM_RADIANS", SQL_FN_NUM_RADIANS},
#endif
#ifdef SQL_FN_NUM_ROUND
    {"FN_NUM_ROUND",  SQL_FN_NUM_ROUND},
#endif
#ifdef SQL_FN_NUM_TRUNCATE
    {"FN_NUM_TRUNCATE", SQL_FN_NUM_TRUNCATE},
#endif
#ifdef SQL_FN_TD_NOW
    {"FN_TD_NOW",     SQL_FN_TD_NOW},
#endif
#ifdef SQL_FN_TD_CURDATE
    {"FN_TD_CURDATE", SQL_FN_TD_CURDATE},
#endif
#ifdef SQL_FN_TD_DAYOFMONTH
    {"FN_TD_DAYOFMONTH", SQL_FN_TD_DAYOFMONTH},
#endif
#ifdef SQL_FN_TD_DAYOFWEEK
    {"FN_TD_DAYOFWEEK", SQL_FN_TD_DAYOFWEEK},
#endif
#ifdef SQL_FN_TD_DAYOFYEAR
    {"FN_TD_DAYOFYEAR", SQL_FN_TD_DAYOFYEAR},
#endif
#ifdef SQL_FN_TD_MONTH
    {"FN_TD_MONTH",   SQL_FN_TD_MONTH},
#endif
#ifdef SQL_FN_TD_QUARTER
    {"FN_TD_QUARTER", SQL_FN_TD_QUARTER},
#endif
#ifdef SQL_FN_TD_WEEK
    {"FN_TD_WEEK",    SQL_FN_TD_WEEK},
#endif
#ifdef SQL_FN_TD_YEAR
    {"FN_TD_YEAR",    SQL_FN_TD_YEAR},
#endif
#ifdef SQL_FN_TD_CURTIME
    {"FN_TD_CURTIME", SQL_FN_TD_CURTIME},
#endif
#ifdef SQL_FN_TD_HOUR
    {"FN_TD_HOUR",    SQL_FN_TD_HOUR},
#endif
#ifdef SQL_FN_TD_MINUTE
    {"FN_TD_MINUTE",  SQL_FN_TD_MINUTE},
#endif
#ifdef SQL_FN_TD_SECOND
    {"FN_TD_SECOND",  SQL_FN_TD_SECOND},
#endif
#ifdef SQL_FN_TD_TIMESTAMPADD
    {"FN_TD_TIMESTAMPADD", SQL_FN_TD_TIMESTAMPADD},
#endif
#ifdef SQL_FN_TD_TIMESTAMPDIFF
    {"FN_TD_TIMESTAMPDIFF", SQL_FN_TD_TIMESTAMPDIFF},
#endif
#ifdef SQL_FN_TD_DAYNAME
    {"FN_TD_DAYNAME", SQL_FN_TD_DAYNAME},
#endif
#ifdef SQL_FN_TD_MONTHNAME
    {"FN_TD_MONTHNAME", SQL_FN_TD_MONTHNAME},
#endif
#ifdef SQL_FN_SYS_USERNAME
    {"FN_SYS_USERNAME", SQL_FN_SYS_USERNAME},
#endif
#ifdef SQL_FN_SYS_DBNAME
    {"FN_SYS_DBNAME", SQL_FN_SYS_DBNAME},
#endif
#ifdef SQL_FN_SYS_IFNULL
    {"FN_SYS_IFNULL", SQL_FN_SYS_IFNULL},
#endif
#ifdef SQL_FN_TSI_FRAC_SECOND
    {"FN_TSI_FRAC_SECOND", SQL_FN_TSI_FRAC_SECOND},
#endif
#ifdef SQL_FN_TSI_SECOND
    {"FN_TSI_SECOND", SQL_FN_TSI_SECOND},
#endif
#ifdef SQL_FN_TSI_MINUTE
    {"FN_TSI_MINUTE", SQL_FN_TSI_MINUTE},
#endif
#ifdef SQL_FN_TSI_HOUR
    {"FN_TSI_HOUR",   SQL_FN_TSI_HOUR},
#endif
#ifdef SQL_FN_TSI_DAY
    {"FN_TSI_DAY",    SQL_FN_TSI_DAY},
#endif
#ifdef SQL_FN_TSI_WEEK
    {"FN_TSI_WEEK",   SQL_FN_TSI_WEEK},
#endif
#ifdef SQL_FN_TSI_MONTH
    {"FN_TSI_MONTH",  SQL_FN_TSI_MONTH},
#endif
#ifdef SQL_FN_TSI_QUARTER
    {"FN_TSI_QUARTER", SQL_FN_TSI_QUARTER},
#endif
#ifdef SQL_FN_TSI_YEAR
    {"FN_TSI_YEAR",   SQL_FN_TSI_YEAR},
#endif
#ifdef SQL_OAC_NONE
    {"OAC_NONE",      SQL_OAC_NONE},
#endif
#ifdef SQL_OAC_LEVEL1
    {"OAC_LEVEL1",    SQL_OAC_LEVEL1},
#endif
#ifdef SQL_OAC_LEVEL2
    {"OAC_LEVEL2",    SQL_OAC_LEVEL2},
#endif
#ifdef SQL_OSCC_NOT_COMPLIANT
    {"OSCC_NOT_COMPLIANT", SQL_OSCC_NOT_COMPLIANT},
#endif
#ifdef SQL_OSCC_COMPLIANT
    {"OSCC_COMPLIANT", SQL_OSCC_COMPLIANT},
#endif
#ifdef SQL_OSC_MINIMUM
    {"OSC_MINIMUM",   SQL_OSC_MINIMUM},
#endif
#ifdef SQL_OSC_CORE
    {"OSC_CORE",      SQL_OSC_CORE},
#endif
#ifdef SQL_OSC_EXTENDED
    {"OSC_EXTENDED",  SQL_OSC_EXTENDED},
#endif
#ifdef SQL_CB_NULL
    {"CB_NULL",       SQL_CB_NULL},
#endif
#ifdef SQL_CB_NON_NULL
    {"CB_NON_NULL",   SQL_CB_NON_NULL},
#endif
#ifdef SQL_CB_DELETE
    {"CB_DELETE",     SQL_CB_DELETE},
#endif
#ifdef SQL_CB_CLOSE
    {"CB_CLOSE",      SQL_CB_CLOSE},
#endif
#ifdef SQL_CB_PRESERVE
    {"CB_PRESERVE",   SQL_CB_PRESERVE},
#endif
#ifdef SQL_IC_UPPER
    {"IC_UPPER",      SQL_IC_UPPER},
#endif
#ifdef SQL_IC_LOWER
    {"IC_LOWER",      SQL_IC_LOWER},
#endif
#ifdef SQL_IC_SENSITIVE
    {"IC_SENSITIVE",  SQL_IC_SENSITIVE},
#endif
#ifdef SQL_IC_MIXED
    {"IC_MIXED",      SQL_IC_MIXED},
#endif
#ifdef SQL_TC_NONE
    {"TC_NONE",       SQL_TC_NONE},
#endif
#ifdef SQL_TC_DML
    {"TC_DML",        SQL_TC_DML},
#endif
#ifdef SQL_TC_ALL
    {"TC_ALL",        SQL_TC_ALL},
#endif
#ifdef SQL_TC_DDL_COMMIT
    {"TC_DDL_COMMIT", SQL_TC_DDL_COMMIT},
#endif
#ifdef SQL_TC_DDL_IGNORE
    {"TC_DDL_IGNORE", SQL_TC_DDL_IGNORE},
#endif
#ifdef SQL_SO_FORWARD_ONLY
    {"SO_FORWARD_ONLY", SQL_SO_FORWARD_ONLY},
#endif
#ifdef SQL_SO_KEYSET_DRIVEN
    {"SO_KEYSET_DRIVEN", SQL_SO_KEYSET_DRIVEN},
#endif
#ifdef SQL_SO_DYNAMIC
    {"SO_DYNAMIC",    SQL_SO_DYNAMIC},
#endif
#ifdef SQL_SO_MIXED
    {"SO_MIXED",      SQL_SO_MIXED},
#endif
#ifdef SQL_SO_STATIC
    {"SO_STATIC",     SQL_SO_STATIC},
#endif
#ifdef SQL_SCCO_READ_ONLY
    {"SCCO_READ_ONLY", SQL_SCCO_READ_ONLY},
#endif
#ifdef SQL_SCCO_LOCK
    {"SCCO_LOCK",     SQL_SCCO_LOCK},
#endif
#ifdef SQL_SCCO_OPT_ROWVER
    {"SCCO_OPT_ROWVER", SQL_SCCO_OPT_ROWVER},
#endif
#ifdef SQL_SCCO_OPT_VALUES
    {"SCCO_OPT_VALUES", SQL_SCCO_OPT_VALUES},
#endif
#ifdef SQL_FD_FETCH_NEXT
    {"FD_FETCH_NEXT", SQL_FD_FETCH_NEXT},
#endif
#ifdef SQL_FD_FETCH_FIRST
    {"FD_FETCH_FIRST", SQL_FD_FETCH_FIRST},
#endif
#ifdef SQL_FD_FETCH_LAST
    {"FD_FETCH_LAST", SQL_FD_FETCH_LAST},
#endif
#ifdef SQL_FD_FETCH_PRIOR
    {"FD_FETCH_PRIOR", SQL_FD_FETCH_PRIOR},
#endif
#ifdef SQL_FD_FETCH_ABSOLUTE
    {"FD_FETCH_ABSOLUTE", SQL_FD_FETCH_ABSOLUTE},
#endif
#ifdef SQL_FD_FETCH_RELATIVE
    {"FD_FETCH_RELATIVE", SQL_FD_FETCH_RELATIVE},
#endif
#ifdef SQL_FD_FETCH_RESUME
    {"FD_FETCH_RESUME", SQL_FD_FETCH_RESUME},
#endif
#ifdef SQL_FD_FETCH_BOOKMARK
    {"FD_FETCH_BOOKMARK", SQL_FD_FETCH_BOOKMARK},
#endif
#ifdef SQL_TXN_READ_UNCOMMITTED
    {"TXN_READ_UNCOMMITTED", SQL_TXN_READ_UNCOMMITTED},
#endif
#ifdef SQL_TXN_READ_COMMITTED
    {"TXN_READ_COMMITTED", SQL_TXN_READ_COMMITTED},
#endif
#ifdef SQL_TXN_REPEATABLE_READ
    {"TXN_REPEATABLE_READ", SQL_TXN_REPEATABLE_READ},
#endif
#ifdef SQL_TXN_SERIALIZABLE
    {"TXN_SERIALIZABLE", SQL_TXN_SERIALIZABLE},
#endif
#ifdef SQL_TXN_VERSIONING
    {"TXN_VERSIONING", SQL_TXN_VERSIONING},
#endif
#ifdef SQL_CN_NONE
    {"CN_NONE",       SQL_CN_NONE},
#endif
#ifdef SQL_CN_DIFFERENT
    {"CN_DIFFERENT",  SQL_CN_DIFFERENT},
#endif
#ifdef SQL_CN_ANY
    {"CN_ANY",        SQL_CN_ANY},
#endif
#ifdef SQL_NNC_NULL
    {"NNC_NULL",      SQL_NNC_NULL},
#endif
#ifdef SQL_NNC_NON_NULL
    {"NNC_NON_NULL",  SQL_NNC_NON_NULL},
#endif
#ifdef SQL_NC_HIGH
    {"NC_HIGH",       SQL_NC_HIGH},
#endif
#ifdef SQL_NC_LOW
    {"NC_LOW",        SQL_NC_LOW},
#endif
#ifdef SQL_NC_START
    {"NC_START",      SQL_NC_START},
#endif
#ifdef SQL_NC_END
    {"NC_END",        SQL_NC_END},
#endif
#ifdef SQL_FILE_NOT_SUPPORTED
    {"FILE_NOT_SUPPORTED", SQL_FILE_NOT_SUPPORTED},
#endif
#ifdef SQL_FILE_TABLE
    {"FILE_TABLE",    SQL_FILE_TABLE},
#endif
#ifdef SQL_FILE_QUALIFIER
    {"FILE_QUALIFIER", SQL_FILE_QUALIFIER},
#endif
#ifdef SQL_GD_ANY_COLUMN
    {"GD_ANY_COLUMN", SQL_GD_ANY_COLUMN},
#endif
#ifdef SQL_GD_ANY_ORDER
    {"GD_ANY_ORDER",  SQL_GD_ANY_ORDER},
#endif
#ifdef SQL_GD_BLOCK
    {"GD_BLOCK",      SQL_GD_BLOCK},
#endif
#ifdef SQL_GD_BOUND
    {"GD_BOUND",      SQL_GD_BOUND},
#endif
#ifdef SQL_AT_ADD_COLUMN
    {"AT_ADD_COLUMN", SQL_AT_ADD_COLUMN},
#endif
#ifdef SQL_AT_DROP_COLUMN
    {"AT_DROP_COLUMN", SQL_AT_DROP_COLUMN},
#endif
#ifdef SQL_PS_POSITIONED_DELETE
    {"PS_POSITIONED_DELETE", SQL_PS_POSITIONED_DELETE},
#endif
#ifdef SQL_PS_POSITIONED_UPDATE
    {"PS_POSITIONED_UPDATE", SQL_PS_POSITIONED_UPDATE},
#endif
#ifdef SQL_PS_SELECT_FOR_UPDATE
    {"PS_SELECT_FOR_UPDATE", SQL_PS_SELECT_FOR_UPDATE},
#endif
#ifdef SQL_GB_NOT_SUPPORTED
    {"GB_NOT_SUPPORTED", SQL_GB_NOT_SUPPORTED},
#endif
#ifdef SQL_GB_GROUP_BY_EQUALS_SELECT
    {"GB_GROUP_BY_EQUALS_SELECT", SQL_GB_GROUP_BY_EQUALS_SELECT},
#endif
#ifdef SQL_GB_GROUP_BY_CONTAINS_SELECT
    {"GB_GROUP_BY_CONTAINS_SELECT", SQL_GB_GROUP_BY_CONTAINS_SELECT},
#endif
#ifdef SQL_GB_NO_RELATION
    {"GB_NO_RELATION", SQL_GB_NO_RELATION},
#endif
#ifdef SQL_OU_DML_STATEMENTS
    {"OU_DML_STATEMENTS", SQL_OU_DML_STATEMENTS},
#endif
#ifdef SQL_OU_PROCEDURE_INVOCATION
    {"OU_PROCEDURE_INVOCATION", SQL_OU_PROCEDURE_INVOCATION},
#endif
#ifdef SQL_OU_TABLE_DEFINITION
    {"OU_TABLE_DEFINITION", SQL_OU_TABLE_DEFINITION},
#endif
#ifdef SQL_OU_INDEX_DEFINITION
    {"OU_INDEX_DEFINITION", SQL_OU_INDEX_DEFINITION},
#endif
#ifdef SQL_OU_PRIVILEGE_DEFINITION
    {"OU_PRIVILEGE_DEFINITION", SQL_OU_PRIVILEGE_DEFINITION},
#endif
#ifdef SQL_QU_DML_STATEMENTS
    {"QU_DML_STATEMENTS", SQL_QU_DML_STATEMENTS},
#endif
#ifdef SQL_QU_PROCEDURE_INVOCATION
    {"QU_PROCEDURE_INVOCATION", SQL_QU_PROCEDURE_INVOCATION},
#endif
#ifdef SQL_QU_TABLE_DEFINITION
    {"QU_TABLE_DEFINITION", SQL_QU_TABLE_DEFINITION},
#endif
#ifdef SQL_QU_INDEX_DEFINITION
    {"QU_INDEX_DEFINITION", SQL_QU_INDEX_DEFINITION},
#endif
#ifdef SQL_QU_PRIVILEGE_DEFINITION
    {"QU_PRIVILEGE_DEFINITION", SQL_QU_PRIVILEGE_DEFINITION},
#endif
#ifdef SQL_SQ_COMPARISON
    {"SQ_COMPARISON", SQL_SQ_COMPARISON},
#endif
#ifdef SQL_SQ_EXISTS
    {"SQ_EXISTS",     SQL_SQ_EXISTS},
#endif
#ifdef SQL_SQ_IN
    {"SQ_IN",         SQL_SQ_IN},
#endif
#ifdef SQL_SQ_QUANTIFIED
    {"SQ_QUANTIFIED", SQL_SQ_QUANTIFIED},
#endif
#ifdef SQL_SQ_CORRELATED_SUBQUERIES
    {"SQ_CORRELATED_SUBQUERIES", SQL_SQ_CORRELATED_SUBQUERIES},
#endif
#ifdef SQL_U_UNION
    {"U_UNION",       SQL_U_UNION},
#endif
#ifdef SQL_U_UNION_ALL
    {"U_UNION_ALL",   SQL_U_UNION_ALL},
#endif
#ifdef SQL_BP_CLOSE
    {"BP_CLOSE",      SQL_BP_CLOSE},
#endif
#ifdef SQL_BP_DELETE
    {"BP_DELETE",     SQL_BP_DELETE},
#endif
#ifdef SQL_BP_DROP
    {"BP_DROP",       SQL_BP_DROP},
#endif
#ifdef SQL_BP_TRANSACTION
    {"BP_TRANSACTION", SQL_BP_TRANSACTION},
#endif
#ifdef SQL_BP_UPDATE
    {"BP_UPDATE",     SQL_BP_UPDATE},
#endif
#ifdef SQL_BP_OTHER_HSTMT
    {"BP_OTHER_HSTMT", SQL_BP_OTHER_HSTMT},
#endif
#ifdef SQL_BP_SCROLL
    {"BP_SCROLL",     SQL_BP_SCROLL},
#endif
#ifdef SQL_SS_ADDITIONS
    {"SS_ADDITIONS",  SQL_SS_ADDITIONS},
#endif
#ifdef SQL_SS_DELETIONS
    {"SS_DELETIONS",  SQL_SS_DELETIONS},
#endif
#ifdef SQL_SS_UPDATES
    {"SS_UPDATES",    SQL_SS_UPDATES},
#endif
#ifdef SQL_LCK_NO_CHANGE
    {"LCK_NO_CHANGE", SQL_LCK_NO_CHANGE},
#endif
#ifdef SQL_LCK_EXCLUSIVE
    {"LCK_EXCLUSIVE", SQL_LCK_EXCLUSIVE},
#endif
#ifdef SQL_LCK_UNLOCK
    {"LCK_UNLOCK",    SQL_LCK_UNLOCK},
#endif
#ifdef SQL_POS_POSITION
    {"POS_POSITION",  SQL_POS_POSITION},
#endif
#ifdef SQL_POS_REFRESH
    {"POS_REFRESH",   SQL_POS_REFRESH},
#endif
#ifdef SQL_POS_UPDATE
    {"POS_UPDATE",    SQL_POS_UPDATE},
#endif
#ifdef SQL_POS_DELETE
    {"POS_DELETE",    SQL_POS_DELETE},
#endif
#ifdef SQL_POS_ADD
    {"POS_ADD",       SQL_POS_ADD},
#endif
#ifdef SQL_QL_START
    {"QL_START",      SQL_QL_START},
#endif
#ifdef SQL_QL_END
    {"QL_END",        SQL_QL_END},
#endif
#ifdef SQL_OJ_LEFT
    {"OJ_LEFT",       SQL_OJ_LEFT},
#endif
#ifdef SQL_OJ_RIGHT
    {"OJ_RIGHT",      SQL_OJ_RIGHT},
#endif
#ifdef SQL_OJ_FULL
    {"OJ_FULL",       SQL_OJ_FULL},
#endif
#ifdef SQL_OJ_NESTED
    {"OJ_NESTED",     SQL_OJ_NESTED},
#endif
#ifdef SQL_OJ_NOT_ORDERED
    {"OJ_NOT_ORDERED", SQL_OJ_NOT_ORDERED},
#endif
#ifdef SQL_OJ_INNER
    {"OJ_INNER",      SQL_OJ_INNER},
#endif
#ifdef SQL_OJ_ALL_COMPARISON_OPS
    {"OJ_ALL_COMPARISON_OPS", SQL_OJ_ALL_COMPARISON_OPS},
#endif
#ifdef SQL_QUERY_TIMEOUT
    {"QUERY_TIMEOUT", SQL_QUERY_TIMEOUT},
#endif
#ifdef SQL_MAX_ROWS
    {"MAX_ROWS",      SQL_MAX_ROWS},
#endif
#ifdef SQL_NOSCAN
    {"NOSCAN",        SQL_NOSCAN},
#endif
#ifdef SQL_MAX_LENGTH
    {"MAX_LENGTH",    SQL_MAX_LENGTH},
#endif
#ifdef SQL_ASYNC_ENABLE
    {"ASYNC_ENABLE",  SQL_ASYNC_ENABLE},
#endif
#ifdef SQL_BIND_TYPE
    {"BIND_TYPE",     SQL_BIND_TYPE},
#endif
#ifdef SQL_CURSOR_TYPE
    {"CURSOR_TYPE",   SQL_CURSOR_TYPE},
#endif
#ifdef SQL_CONCURRENCY
    {"CONCURRENCY",   SQL_CONCURRENCY},
#endif
#ifdef SQL_KEYSET_SIZE
    {"KEYSET_SIZE",   SQL_KEYSET_SIZE},
#endif
#ifdef SQL_ROWSET_SIZE
    {"ROWSET_SIZE",   SQL_ROWSET_SIZE},
#endif
#ifdef SQL_SIMULATE_CURSOR
    {"SIMULATE_CURSOR", SQL_SIMULATE_CURSOR},
#endif
#ifdef SQL_RETRIEVE_DATA
    {"RETRIEVE_DATA", SQL_RETRIEVE_DATA},
#endif
#ifdef SQL_USE_BOOKMARKS
    {"USE_BOOKMARKS", SQL_USE_BOOKMARKS},
#endif
#ifdef SQL_GET_BOOKMARK
    {"GET_BOOKMARK",  SQL_GET_BOOKMARK}, /* GetStmtOption Only */
#endif
#ifdef SQL_ROW_NUMBER
    {"ROW_NUMBER",    SQL_ROW_NUMBER}, /* GetStmtOption Only */
#endif
#ifdef SQL_STMT_OPT_MAX
    {"STMT_OPT_MAX",  SQL_STMT_OPT_MAX},
#endif
#ifdef SQL_STMT_OPT_MIN
    {"STMT_OPT_MIN",  SQL_STMT_OPT_MIN},
#endif
#ifdef SQL_QUERY_TIMEOUT_DEFAULT
    {"QUERY_TIMEOUT_DEFAULT", SQL_QUERY_TIMEOUT_DEFAULT},
#endif
#ifdef SQL_MAX_ROWS_DEFAULT
    {"MAX_ROWS_DEFAULT", SQL_MAX_ROWS_DEFAULT},
#endif
#ifdef SQL_NOSCAN_OFF
    {"NOSCAN_OFF",    SQL_NOSCAN_OFF}, /* 1.0 FALSE */
#endif
#ifdef SQL_NOSCAN_ON
    {"NOSCAN_ON",     SQL_NOSCAN_ON}, /* 1.0 TRUE */
#endif
#ifdef SQL_NOSCAN_DEFAULT
    {"NOSCAN_DEFAULT", SQL_NOSCAN_DEFAULT},
#endif
#ifdef SQL_MAX_LENGTH_DEFAULT
    {"MAX_LENGTH_DEFAULT", SQL_MAX_LENGTH_DEFAULT},
#endif
#ifdef SQL_ASYNC_ENABLE_OFF
    {"ASYNC_ENABLE_OFF", SQL_ASYNC_ENABLE_OFF},
#endif
#ifdef SQL_ASYNC_ENABLE_ON
    {"ASYNC_ENABLE_ON", SQL_ASYNC_ENABLE_ON},
#endif
#ifdef SQL_ASYNC_ENABLE_DEFAULT
    {"ASYNC_ENABLE_DEFAULT", SQL_ASYNC_ENABLE_DEFAULT},
#endif
#ifdef SQL_BIND_BY_COLUMN
    {"BIND_BY_COLUMN", SQL_BIND_BY_COLUMN},
#endif
#ifdef SQL_BIND_TYPE_DEFAULT
    {"BIND_TYPE_DEFAULT", SQL_BIND_TYPE_DEFAULT}, /* Default value */
#endif
#ifdef SQL_CONCUR_READ_ONLY
    {"CONCUR_READ_ONLY", SQL_CONCUR_READ_ONLY},
#endif
#ifdef SQL_CONCUR_LOCK
    {"CONCUR_LOCK",   SQL_CONCUR_LOCK},
#endif
#ifdef SQL_CONCUR_ROWVER
    {"CONCUR_ROWVER", SQL_CONCUR_ROWVER},
#endif
#ifdef SQL_CONCUR_VALUES
    {"CONCUR_VALUES", SQL_CONCUR_VALUES},
#endif
#ifdef SQL_CONCUR_DEFAULT
    {"CONCUR_DEFAULT", SQL_CONCUR_DEFAULT}, /* Default value */
#endif
#ifdef SQL_CURSOR_FORWARD_ONLY
    {"CURSOR_FORWARD_ONLY", SQL_CURSOR_FORWARD_ONLY},
#endif
#ifdef SQL_CURSOR_KEYSET_DRIVEN
    {"CURSOR_KEYSET_DRIVEN", SQL_CURSOR_KEYSET_DRIVEN},
#endif
#ifdef SQL_CURSOR_DYNAMIC
    {"CURSOR_DYNAMIC", SQL_CURSOR_DYNAMIC},
#endif
#ifdef SQL_CURSOR_STATIC
    {"CURSOR_STATIC", SQL_CURSOR_STATIC},
#endif
#ifdef SQL_CURSOR_TYPE_DEFAULT
    {"CURSOR_TYPE_DEFAULT", SQL_CURSOR_TYPE_DEFAULT}, /* Default value */
#endif
#ifdef SQL_ROWSET_SIZE_DEFAULT
    {"ROWSET_SIZE_DEFAULT", SQL_ROWSET_SIZE_DEFAULT},
#endif
#ifdef SQL_KEYSET_SIZE_DEFAULT
    {"KEYSET_SIZE_DEFAULT", SQL_KEYSET_SIZE_DEFAULT},
#endif
#ifdef SQL_SC_NON_UNIQUE
    {"SC_NON_UNIQUE", SQL_SC_NON_UNIQUE},
#endif
#ifdef SQL_SC_TRY_UNIQUE
    {"SC_TRY_UNIQUE", SQL_SC_TRY_UNIQUE},
#endif
#ifdef SQL_SC_UNIQUE
    {"SC_UNIQUE",     SQL_SC_UNIQUE},
#endif
#ifdef SQL_RD_OFF
    {"RD_OFF",        SQL_RD_OFF},
#endif
#ifdef SQL_RD_ON
    {"RD_ON",         SQL_RD_ON},
#endif
#ifdef SQL_RD_DEFAULT
    {"RD_DEFAULT",    SQL_RD_DEFAULT},
#endif
#ifdef SQL_UB_OFF
    {"UB_OFF",        SQL_UB_OFF},
#endif
#ifdef SQL_UB_ON
    {"UB_ON",         SQL_UB_ON},
#endif
#ifdef SQL_UB_DEFAULT
    {"UB_DEFAULT",    SQL_UB_DEFAULT},
#endif
#ifdef SQL_ACCESS_MODE
    {"ACCESS_MODE",   SQL_ACCESS_MODE},
#endif
#ifdef SQL_AUTOCOMMIT
    {"AUTOCOMMIT",    SQL_AUTOCOMMIT},
#endif
#ifdef SQL_LOGIN_TIMEOUT
    {"LOGIN_TIMEOUT", SQL_LOGIN_TIMEOUT},
#endif
#ifdef SQL_OPT_TRACE
    {"OPT_TRACE",     SQL_OPT_TRACE},
#endif
#ifdef SQL_OPT_TRACEFILE
    {"OPT_TRACEFILE", SQL_OPT_TRACEFILE},
#endif
#ifdef SQL_TRANSLATE_DLL
    {"TRANSLATE_DLL", SQL_TRANSLATE_DLL},
#endif
#ifdef SQL_TRANSLATE_OPTION
    {"TRANSLATE_OPTION", SQL_TRANSLATE_OPTION},
#endif
#ifdef SQL_TXN_ISOLATION
    {"TXN_ISOLATION", SQL_TXN_ISOLATION},
#endif
#ifdef SQL_CURRENT_QUALIFIER
    {"CURRENT_QUALIFIER", SQL_CURRENT_QUALIFIER},
#endif
#ifdef SQL_ODBC_CURSORS
    {"ODBC_CURSORS",  SQL_ODBC_CURSORS},
#endif
#ifdef SQL_QUIET_MODE
    {"QUIET_MODE",    SQL_QUIET_MODE},
#endif
#ifdef API_SQL_PACKET_SIZE
    {"API_SQL_PACKET_SIZE", API_SQL_PACKET_SIZE},
#endif
#ifdef SQL_CONN_OPT_MAX
    {"CONN_OPT_MAX",  SQL_CONN_OPT_MAX},
#endif
#ifdef SQL_CONNECT_OPT_DRVR_START
    {"CONNECT_OPT_DRVR_START", SQL_CONNECT_OPT_DRVR_START},
#endif
#ifdef SQL_CONN_OPT_MIN
    {"CONN_OPT_MIN",  SQL_CONN_OPT_MIN},
#endif
#ifdef SQL_MODE_READ_WRITE
    {"MODE_READ_WRITE", SQL_MODE_READ_WRITE},
#endif
#ifdef SQL_MODE_READ_ONLY
    {"MODE_READ_ONLY", SQL_MODE_READ_ONLY},
#endif
#ifdef SQL_MODE_DEFAULT
    {"MODE_DEFAULT",  SQL_MODE_DEFAULT},
#endif
#ifdef SQL_AUTOCOMMIT_OFF
    {"AUTOCOMMIT_OFF", SQL_AUTOCOMMIT_OFF},
#endif
#ifdef SQL_AUTOCOMMIT_ON
    {"AUTOCOMMIT_ON", SQL_AUTOCOMMIT_ON},
#endif
#ifdef SQL_AUTOCOMMIT_DEFAULT
    {"AUTOCOMMIT_DEFAULT", SQL_AUTOCOMMIT_DEFAULT},
#endif
#ifdef SQL_LOGIN_TIMEOUT_DEFAULT
    {"LOGIN_TIMEOUT_DEFAULT", SQL_LOGIN_TIMEOUT_DEFAULT},
#endif
#ifdef SQL_OPT_TRACE_OFF
    {"OPT_TRACE_OFF", SQL_OPT_TRACE_OFF},
#endif
#ifdef SQL_OPT_TRACE_ON
    {"OPT_TRACE_ON",  SQL_OPT_TRACE_ON},
#endif
#ifdef SQL_OPT_TRACE_DEFAULT
    {"OPT_TRACE_DEFAULT", SQL_OPT_TRACE_DEFAULT},
#endif
#ifdef SQL_CUR_USE_IF_NEEDED
    {"CUR_USE_IF_NEEDED", SQL_CUR_USE_IF_NEEDED},
#endif
#ifdef SQL_CUR_USE_ODBC
    {"CUR_USE_ODBC",  SQL_CUR_USE_ODBC},
#endif
#ifdef SQL_CUR_USE_DRIVER
    {"CUR_USE_DRIVER", SQL_CUR_USE_DRIVER},
#endif
#ifdef SQL_CUR_DEFAULT
    {"CUR_DEFAULT",   SQL_CUR_DEFAULT},
#endif
#ifdef SQL_BEST_ROWID
    {"BEST_ROWID",    SQL_BEST_ROWID},
#endif
#ifdef SQL_ROWVER
    {"ROWVER",        SQL_ROWVER},
#endif
#ifdef SQL_SCOPE_CURROW
    {"SCOPE_CURROW",  SQL_SCOPE_CURROW},
#endif
#ifdef SQL_SCOPE_TRANSACTION
    {"SCOPE_TRANSACTION", SQL_SCOPE_TRANSACTION},
#endif
#ifdef SQL_SCOPE_SESSION
    {"SCOPE_SESSION", SQL_SCOPE_SESSION},
#endif
#ifdef SQL_ENTIRE_ROWSET
    {"ENTIRE_ROWSET", SQL_ENTIRE_ROWSET},
#endif
#ifdef SQL_POSITION
    {"POSITION",      SQL_POSITION}, /* 1.0 FALSE */
#endif
#ifdef SQL_REFRESH
    {"REFRESH",       SQL_REFRESH}, /* 1.0 TRUE */
#endif
#ifdef SQL_UPDATE
    {"UPDATE",        SQL_UPDATE},
#endif
#ifdef SQL_DELETE
    {"DELETE",        SQL_DELETE},
#endif
#ifdef SQL_ADD
    {"ADD",           SQL_ADD},
#endif
#ifdef SQL_LOCK_NO_CHANGE
    {"LOCK_NO_CHANGE", SQL_LOCK_NO_CHANGE}, /* 1.0 FALSE */
#endif
#ifdef SQL_LOCK_EXCLUSIVE
    {"LOCK_EXCLUSIVE", SQL_LOCK_EXCLUSIVE}, /* 1.0 TRUE */
#endif
#ifdef SQL_LOCK_UNLOCK
    {"LOCK_UNLOCK",   SQL_LOCK_UNLOCK},
#endif
#ifdef SQL_POSITION_T
    {"POSITION_T",    SQL_POSITION_T},
#endif
#ifdef SQL_LOCK_RECOR
    {"LOCK_RECOR",    SQL_LOCK_RECOR},
#endif
#ifdef SQL_REFRESH_RECOR
    {"REFRESH_RECOR", SQL_REFRESH_RECOR},
#endif
#ifdef SQL_UPDATE_RECOR
    {"UPDATE_RECOR",  SQL_UPDATE_RECOR},
#endif
#ifdef SQL_DELETE_RECOR
    {"DELETE_RECOR",  SQL_DELETE_RECOR},
#endif
#ifdef SQL_ADD_RECOR
    {"ADD_RECOR",     SQL_ADD_RECOR},
#endif
#ifdef SQL_FETCH_NEXT
    {"FETCH_NEXT",    SQL_FETCH_NEXT},
#endif
#ifdef SQL_FETCH_FIRST
    {"FETCH_FIRST",   SQL_FETCH_FIRST},
#endif
#ifdef SQL_FETCH_LAST
    {"FETCH_LAST",    SQL_FETCH_LAST},
#endif
#ifdef SQL_FETCH_PRIOR
    {"FETCH_PRIOR",   SQL_FETCH_PRIOR},
#endif
#ifdef SQL_FETCH_ABSOLUTE
    {"FETCH_ABSOLUTE", SQL_FETCH_ABSOLUTE},
#endif
#ifdef SQL_FETCH_RELATIVE
    {"FETCH_RELATIVE", SQL_FETCH_RELATIVE},
#endif
#ifdef SQL_FETCH_BOOKMARK
    {"FETCH_BOOKMARK", SQL_FETCH_BOOKMARK},
#endif
#ifdef SQL_ROW_SUCCESS
    {"ROW_SUCCESS",   SQL_ROW_SUCCESS},
#endif
#ifdef SQL_ROW_DELETED
    {"ROW_DELETED",   SQL_ROW_DELETED},
#endif
#ifdef SQL_ROW_UPDATED
    {"ROW_UPDATED",   SQL_ROW_UPDATED},
#endif
#ifdef SQL_ROW_NOROW
    {"ROW_NOROW",     SQL_ROW_NOROW},
#endif
#ifdef SQL_ROW_ADDED
    {"ROW_ADDED",     SQL_ROW_ADDED},
#endif
#ifdef SQL_ROW_ERROR
    {"ROW_ERROR",     SQL_ROW_ERROR},
#endif
#ifdef SQL_CASCADE
    {"CASCADE",       SQL_CASCADE},
#endif
#ifdef SQL_RESTRICT
    {"RESTRICT",      SQL_RESTRICT},
#endif
#ifdef SQL_SET_NULL
    {"SET_NULL",      SQL_SET_NULL},
#endif
#ifdef SQL_PARAM_TYPE_UNKNOWN
    {"PARAM_TYPE_UNKNOWN", SQL_PARAM_TYPE_UNKNOWN},
#endif
#ifdef SQL_PARAM_INPUT
    {"PARAM_INPUT",   SQL_PARAM_INPUT},
#endif
#ifdef SQL_PARAM_INPUT_OUTPUT
    {"PARAM_INPUT_OUTPUT", SQL_PARAM_INPUT_OUTPUT},
#endif
#ifdef SQL_RESULT_COL
    {"RESULT_COL",    SQL_RESULT_COL},
#endif
#ifdef SQL_PARAM_OUTPUT
    {"PARAM_OUTPUT",  SQL_PARAM_OUTPUT},
#endif
#ifdef SQL_RETURN_VALUE
    {"RETURN_VALUE",  SQL_RETURN_VALUE},
#endif
#ifdef SQL_PARAM_TYPE_DEFAULT
    {"PARAM_TYPE_DEFAULT", SQL_PARAM_TYPE_DEFAULT},
#endif
#ifdef SQL_SETPARAM_VALUE_MAX
    {"SETPARAM_VALUE_MAX", SQL_SETPARAM_VALUE_MAX},
#endif
#ifdef SQL_INDEX_UNIQUE
    {"INDEX_UNIQUE",  SQL_INDEX_UNIQUE},
#endif
#ifdef SQL_INDEX_ALL
    {"INDEX_ALL",     SQL_INDEX_ALL},
#endif
#ifdef SQL_QUICK
    {"QUICK",         SQL_QUICK},
#endif
#ifdef SQL_ENSURE
    {"ENSURE",        SQL_ENSURE},
#endif
#ifdef SQL_TABLE_STAT
    {"TABLE_STAT",    SQL_TABLE_STAT},
#endif
#ifdef SQL_INDEX_CLUSTERED
    {"INDEX_CLUSTERED", SQL_INDEX_CLUSTERED},
#endif
#ifdef SQL_INDEX_HASHED
    {"INDEX_HASHED",  SQL_INDEX_HASHED},
#endif
#ifdef SQL_INDEX_OTHER
    {"INDEX_OTHER",   SQL_INDEX_OTHER},
#endif
#ifdef SQL_PT_UNKNOWN
    {"PT_UNKNOWN",    SQL_PT_UNKNOWN},
#endif
#ifdef SQL_PT_PROCEDURE
    {"PT_PROCEDURE",  SQL_PT_PROCEDURE},
#endif
#ifdef SQL_PT_FUNCTION
    {"PT_FUNCTION",   SQL_PT_FUNCTION},
#endif
#ifdef SQL_PC_UNKNOWN
    {"PC_UNKNOWN",    SQL_PC_UNKNOWN},
#endif
#ifdef SQL_PC_NOT_PSEUDO
    {"PC_NOT_PSEUDO", SQL_PC_NOT_PSEUDO},
#endif
#ifdef SQL_PC_PSEUDO
    {"PC_PSEUDO",     SQL_PC_PSEUDO},
#endif
#ifdef SQL_DATABASE_NAME
    {"DATABASE_NAME", SQL_DATABASE_NAME}, /* Use SQLGetConnectOption/SQL_CURRENT_QUALIFIER */
#endif
#ifdef SQL_FD_FETCH_PREV
    {"FD_FETCH_PREV", SQL_FD_FETCH_PREV},
#endif
#ifdef SQL_FETCH_PREV
    {"FETCH_PREV",    SQL_FETCH_PREV},
#endif
#ifdef SQL_CONCUR_TIMESTAMP
    {"CONCUR_TIMESTAMP", SQL_CONCUR_TIMESTAMP},
#endif
#ifdef SQL_SCCO_OPT_TIMESTAMP
    {"SCCO_OPT_TIMESTAMP", SQL_SCCO_OPT_TIMESTAMP},
#endif
#ifdef SQL_CC_DELETE
    {"CC_DELETE",     SQL_CC_DELETE},
#endif
#ifdef SQL_CR_DELETE
    {"CR_DELETE",     SQL_CR_DELETE},
#endif
#ifdef SQL_CC_CLOSE
    {"CC_CLOSE",      SQL_CC_CLOSE},
#endif
#ifdef SQL_CR_CLOSE
    {"CR_CLOSE",      SQL_CR_CLOSE},
#endif
#ifdef SQL_CC_PRESERVE
    {"CC_PRESERVE",   SQL_CC_PRESERVE},
#endif
#ifdef SQL_CR_PRESERVE
    {"CR_PRESERVE",   SQL_CR_PRESERVE},
#endif
#ifdef SQL_FETCH_RESUME
    {"FETCH_RESUME",  SQL_FETCH_RESUME}, /* Not supported by 2.0 drivers */
#endif
#ifdef SQL_SCROLL_FORWARD_ONLY
    {"SCROLL_FORWARD_ONLY", SQL_SCROLL_FORWARD_ONLY}, /* -SQL_CURSOR_FORWARD_ONLY */
#endif
#ifdef SQL_SCROLL_KEYSET_DRIVEN
    {"SCROLL_KEYSET_DRIVEN", SQL_SCROLL_KEYSET_DRIVEN},
#endif
#ifdef SQL_SCROLL_DYNAMIC
    {"SCROLL_DYNAMIC", SQL_SCROLL_DYNAMIC},
#endif
#ifdef SQL_SCROLL_STATIC
    {"SCROLL_STATIC", SQL_SCROLL_STATIC},
#endif
#ifdef SQL_PC_NON_PSEUDO
    {"PC_NON_PSEUDO", SQL_PC_NON_PSEUDO},
#endif
};

/* --- API ---------------------------------------------------------------- */

PyObject *mxODBC_SQL_Class(void)
{
    PyObject *d, *c;
    int i;
    
    d = PyDict_New();
    if (d == NULL)
	goto onError;
    for (i = 0; i < sizeof(mxODBC_SQLDefines) /
	            sizeof(mxODBC_SQLDefine); i++) {
	PyObject *v,*w;
	v = PyString_FromString(mxODBC_SQLDefines[i].sqlstring);
	if (v == NULL)
	    goto onError;
	w = PyInt_FromLong((long)mxODBC_SQLDefines[i].sqlcode);
	if (w == NULL)
	    goto onError;
	if (PyDict_SetItem(d,v,w))
	    goto onError;
    }
#ifdef INTERN_STRINGS
    c = PyClass_New(NULL,d,PyString_InternFromString("SQL"));
#else
    c = PyClass_New(NULL,d,PyString_FromString("SQL"));
#endif
    if (c == NULL)
	goto onError;
    return c;
    
 onError:
    return NULL;
}
