""" Generic APIs useful for mxODBC.

    (c) Copyright Marc-Andre Lemburg; All Rights Reserved.
    See the documentation for further information on copyrights,
    or contact the author (mal@lemburg.com).

"""
import string

def format_resultset(cursor,headers=(),colsep=' | ',headersep='-',
		     stringify=repr):

    """ Fetch the result set from cursor and format it into a list
        of strings (one for each row):

	<header>
	<headersep>
	<row1>
	<row2>
	...

	headers may be given as list of strings. If not given, or too
	short, the function will add numbered columns as appropriate.

	Columns are separated by colsep; the header is separated from
	the result set by a line of headersep characters.

	The function calls stringify to format the value data returned
	by the driver into a string. It defauls to repr().

    """
    # Get result set
    rows = cursor.fetchall()
    colcount = len(cursor.description)

    # Check row entry lengths
    output = []
    headers = list(headers)
    if len(headers) < colcount:
	for i in range(len(headers),colcount):
	    headers.append('Column %i' % (i + 1))
    collen = map(len,headers)
    output.append(headers)
    rows.sort()
    for row in rows:
	row = map(stringify,row)
	for i in range(len(row)):
	    entry = row[i]
	    if collen[i] < len(entry):
		collen[i] = len(entry)
	output.append(row)

    # Format output
    for i in range(len(output)):
	row = output[i]
	l = []
	for j in range(len(row)):
	    l.append('%-*s' % (collen[j],row[j]))
	output[i] = string.join(l,colsep)

    # Insert header separator
    totallen = len(output[0])
    output[1:1] = [headersep*(totallen / len(headersep))]

    return output

def print_resultset(cursor,headers=(),**kws):

    """ Pretty-prints the current result set available through
        cursor.

	headers can be given as list of column header strings.

    """
    lines = apply(format_resultset,(cursor,headers),kws)
    for line in lines:
	print line
